# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2016, Alex Băluț <alexandru.balut@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the render module."""
# pylint: disable=protected-access,no-self-use
import os
import shutil
import tempfile
from unittest import mock
from unittest import skipUnless

from gi.repository import GES
from gi.repository import Gst
from gi.repository import GstPbutils
from gi.repository import Gtk

from pitivi.render import Encoders
from pitivi.render import extension_for_muxer
from pitivi.render import PresetsManager
from pitivi.render import Quality
from pitivi.render import quality_adapters
from pitivi.render import QualityAdapter
from pitivi.timeline.timeline import TimelineContainer
from pitivi.utils.proxy import ProxyingStrategy
from pitivi.utils.ui import get_combo_value
from pitivi.utils.ui import set_combo_value
from tests import common
from tests.test_medialibrary import BaseTestMediaLibrary


def factory_exists(*factories):
    """Checks if @factories exists."""
    for factory in factories:
        if not Gst.ElementFactory.find(factory):
            return False, "%s not present on the system" % (factory)

    return True, ""


def encoding_target_exists(tname):
    """Checks if a GstEncodingTarget called @name exists."""
    for target in GstPbutils.encoding_list_all_targets():
        if tname in target.get_name().split(";"):
            return True, ""
    return False, "EncodingTarget %s not present on the system" % tname


def setup_render_presets(*profiles):
    """Temporary directory setup for testing render profiles."""
    def setup_wrapper(func):

        def wrapped(self):
            with tempfile.TemporaryDirectory() as tmp_presets_dir:
                os.mkdir(os.path.join(tmp_presets_dir, "test"))

                for profile in profiles:
                    path = os.path.join(os.environ["PITIVI_TOP_LEVEL_DIR"], "tests/test-encoding-targets/test", profile + ".gep")
                    tmp_path = os.path.join(tmp_presets_dir, "test", profile + ".gep")
                    shutil.copy(path, tmp_path)

                os.environ["GST_ENCODING_TARGET_PATH"] = tmp_presets_dir
                func(self)

        return wrapped

    return setup_wrapper


class TestPresetsManager(common.TestCase):

    @skipUnless(*encoding_target_exists("youtube"))
    @skipUnless(*factory_exists("x264enc"))
    def test_initial_preset(self):
        project = common.create_project()
        manager = PresetsManager(project)

        self.assertEqual(manager.initial_preset().name, "youtube")

    def test_missing_x264(self):
        # Simulate no encoder being available for the profile's format.
        targets = GstPbutils.encoding_list_all_targets()
        for target in targets:
            for profile in target.get_profiles():
                for sub_profile in profile.get_profiles():
                    raw_caps = "audio/non_existing_whatever_it_s_true"
                    sub_profile.get_format = mock.Mock(return_value=raw_caps)

        with mock.patch.object(GstPbutils, "encoding_list_all_targets") as encoding_list_all_targets:
            encoding_list_all_targets.return_value = targets

            project = common.create_project()
            manager = PresetsManager(project)

            self.assertIsNone(manager.initial_preset())


class TestQualityAdapter(common.TestCase):
    """Tests for the QualityAdapter class."""

    def check_adapter(self, adapter, expected_qualities):
        qualities = []
        for prop_value in range(len(expected_qualities)):
            vcodecsettings = {adapter.prop_name: prop_value}
            quality = adapter.calculate_quality(vcodecsettings)
            qualities.append(quality)
        self.assertListEqual(qualities, expected_qualities)

    def test_calculate_quality(self):
        self.check_adapter(QualityAdapter({"prop1": (0, 3, 5)}),
                           [Quality.LOW, Quality.LOW, Quality.LOW, Quality.MEDIUM, Quality.MEDIUM, Quality.HIGH])
        self.check_adapter(QualityAdapter({"prop1": (100, 3, 2)}),
                           [Quality.HIGH, Quality.HIGH, Quality.HIGH, Quality.MEDIUM, Quality.LOW, Quality.LOW])

        self.check_adapter(quality_adapters[Encoders.X264],
                           [Quality.HIGH] * 19 + [Quality.MEDIUM] * 3 + [Quality.LOW] * 29)
        self.check_adapter(quality_adapters[Encoders.VP8],
                           [Quality.LOW] * 47 + [Quality.MEDIUM] * 16 + [Quality.HIGH] * 1)
        self.check_adapter(quality_adapters[Encoders.THEORA],
                           [Quality.LOW] * 48 + [Quality.MEDIUM] * 15 + [Quality.HIGH] * 1)
        self.check_adapter(quality_adapters[Encoders.JPEG],
                           [Quality.LOW] * 85 + [Quality.MEDIUM] * 15 + [Quality.HIGH] * 1)


class TestRender(BaseTestMediaLibrary):
    """Tests for functions."""

    def test_extensions_supported(self):
        """Checks we associate file extensions to the well supported muxers."""
        for muxer, unused_audio, unused_video in Encoders.SUPPORTED_ENCODERS_COMBINATIONS:
            self.assertIsNotNone(extension_for_muxer(muxer), muxer)

    def test_extensions_presets(self):
        """Checks we associate file extensions to the muxers of the presets."""
        project = self.create_simple_project()
        with mock.patch("pitivi.preset.xdg_data_home") as xdg_data_home:
            xdg_data_home.return_value = "/pitivi-dir-which-does-not-exist"
            preset_manager = PresetsManager(project.app)
            preset_manager.load_all()
            for preset_item in preset_manager.model:
                # Preset name is only set when the project loads it
                project.set_container_profile(preset_item.profile)
                muxer = preset_item.profile.get_preset_name()
                self.assertIsNotNone(extension_for_muxer(muxer), preset_item.profile)

    def create_simple_project(self):
        """Creates a Project with a layer a clip."""
        timeline_container = common.create_timeline_container()
        app = timeline_container.app
        project = app.project_manager.current_project
        if not project.ges_timeline.get_layers():
            project.ges_timeline.append_layer()

        mainloop = common.create_main_loop()

        def asset_added_cb(project, asset):
            mainloop.quit()

        project.connect("asset-added", asset_added_cb)
        uris = [common.get_sample_uri("tears_of_steel.webm")]
        project.add_uris(uris)
        mainloop.run()

        layer, = project.ges_timeline.get_layers()
        asset, = project.list_assets(GES.UriClip)
        layer.add_asset(asset, 0, 0, Gst.CLOCK_TIME_NONE, GES.TrackType.UNKNOWN)

        return project

    def create_rendering_dialog(self, project):
        """Creates a RenderingDialog ready for testing."""
        from pitivi.render import RenderDialog

        class MockedBuilder(Gtk.Builder):
            """Specialized builder suitable for RenderingDialog testing."""

            # pylint: disable=arguments-differ
            def get_object(self, name):
                """Get @name widget or a MagicMock for render dialog window."""
                if name == "render-dialog":
                    return mock.MagicMock()

                return super().get_object(name)

        with mock.patch.object(Gtk.Builder, "__new__", return_value=MockedBuilder()):
            return RenderDialog(project.app, project)

    def test_launching_rendering(self):
        """Checks no exception is raised when clicking the render button."""
        project = self.create_simple_project()
        dialog = self.create_rendering_dialog(project)

        from pitivi.render import RenderingProgressDialog
        with mock.patch.object(dialog, "start_action"):
            with mock.patch.object(RenderingProgressDialog, "__new__"):
                with mock.patch.object(dialog, "_pipeline"):
                    return dialog._render_button_clicked_cb(None)

    @skipUnless(*factory_exists("x264enc", "matroskamux"))
    def test_encoder_restrictions(self):
        """Checks the mechanism to respect encoder specific restrictions."""
        project = self.create_simple_project()
        dialog = self.create_rendering_dialog(project)

        # Explicitly set the encoder
        self.assertTrue(set_combo_value(dialog.muxer_combo,
                                        Gst.ElementFactory.find("matroskamux")))
        self.assertTrue(set_combo_value(dialog.video_encoder_combo,
                                        Gst.ElementFactory.find("x264enc")))

        # Set encoding profile
        if getattr(GstPbutils.EncodingProfile, "copy"):  # Available only in > 1.11
            profile = project.container_profile.copy()
            vprofile, = [p for p in profile.get_profiles()
                         if isinstance(p, GstPbutils.EncodingVideoProfile)]
            vprofile.set_restriction(Gst.Caps("video/x-raw"))
            project.set_container_profile(profile)

    @skipUnless(*factory_exists("vorbisenc", "theoraenc", "oggmux",
                                "opusenc", "vp8enc"))
    @setup_render_presets("test")
    def test_loading_preset(self):
        """Checks preset values are properly exposed in the UI."""
        project = self.create_simple_project()
        dialog = self.create_rendering_dialog(project)

        test_data = [
            ("test", {"aencoder": "vorbisenc",
                      "vencoder": "theoraenc",
                      "muxer": "oggmux"}),
            ("test_ogg-vp8-opus", {
                "aencoder": "opusenc",
                "vencoder": ["vp8enc", "vaapivp8enc"],
                "muxer": "oggmux"}),
            ("test_fullhd", {
                "aencoder": "vorbisenc",
                "vencoder": "theoraenc",
                "muxer": "oggmux",
                "videowidth": 1920,
                "videoheight": 1080,
                "videorate": Gst.Fraction(120, 1)}),
            ("test_ogg-vp8-opus", {
                "aencoder": "opusenc",
                "vencoder": ["vp8enc", "vaapivp8enc"],
                "muxer": "oggmux"}),
            ("test_fullhd", {
                "aencoder": "vorbisenc",
                "vencoder": "theoraenc",
                "muxer": "oggmux",
                "videowidth": 1920,
                "videoheight": 1080,
                "videorate": Gst.Fraction(120, 1)}),
        ]

        attr_dialog_widget_map = {
            "videorate": dialog.frame_rate_combo,
            "aencoder": dialog.audio_encoder_combo,
            "vencoder": dialog.video_encoder_combo,
            "muxer": dialog.muxer_combo,
        }

        dialog._preset_selection_menubutton_clicked_cb(None)

        for preset_name, values in test_data:
            self.select_render_preset(dialog, preset_name)

            for attr, val in values.items():
                val = val if isinstance(val, list) else [val]
                combo = attr_dialog_widget_map.get(attr)
                if combo:
                    combo_value = get_combo_value(combo)
                    if isinstance(combo_value, Gst.ElementFactory):
                        combo_value = combo_value.get_name()
                    self.assertIn(combo_value, val, preset_name)

                self.assertIn(getattr(project, attr), val)

    @skipUnless(*factory_exists("vorbisenc", "theoraenc", "oggmux",
                                "opusenc", "vp8enc"))
    @setup_render_presets("test-remove")
    def test_remove_profile(self):
        """Tests removing EncodingProfile and re-saving it."""
        project = self.create_simple_project()
        dialog = self.create_rendering_dialog(project)
        self.select_render_preset(dialog, "test-remove")

        # Check the "test" profile is selected
        self.assertEqual(dialog.presets_manager.cur_preset_item.name, "test-remove")

        # If EncodingTarget has single profile, PresetItem's name is same as that of the EncodingTarget.
        profile_name = dialog.presets_manager.cur_preset_item.name

        if self.assertEqual(profile_name, "test-remove"):
            # Remove current profile and verify it has been removed
            dialog.presets_manager.action_remove.activate()
            self.assertIsNone(dialog.presets_manager.cur_preset_item)
            self.assertEqual(dialog.preset_label.get_text(), "Custom")

    def check_simple_rendering_profile(self, profile_name=None):
        """Checks that rendering with the specified profile works."""
        project = self.create_simple_project()
        dialog = self.create_rendering_dialog(project)
        if profile_name:
            self.select_render_preset(dialog, profile_name)
        self.render(dialog)

    def render(self, dialog):
        """Renders pipeline from @dialog."""
        from pitivi.render import RenderingProgressDialog
        with tempfile.TemporaryDirectory() as temp_dir:
            # Start rendering
            with mock.patch.object(dialog.fileentry, "get_text",
                                   return_value=os.path.join(temp_dir, "outfile")):
                with mock.patch.object(RenderingProgressDialog, "__new__"):
                    dialog._render_button_clicked_cb(None)

            message = dialog._pipeline.get_bus().timed_pop_filtered(
                Gst.CLOCK_TIME_NONE,
                Gst.MessageType.EOS | Gst.MessageType.ERROR)
            self.assertIsNotNone(message)
            Gst.debug_bin_to_dot_file_with_ts(
                dialog._pipeline, Gst.DebugGraphDetails.ALL,
                "test_rendering_with_profile.dot")

            struct = message.get_structure()
            self.assertEqual(message.type, Gst.MessageType.EOS,
                             struct.to_string() if struct else message)

            result_file = Gst.filename_to_uri(os.path.join(temp_dir, "outfile"))
            asset = GES.UriClipAsset.request_sync(result_file)
            self.assertIsNotNone(asset)

            if message:
                dialog._pipeline.get_bus().post(message)

    def test_rendering_with_scale(self):
        """Tests rendering with a smaller scale."""
        sample_name = "30fps_numeroted_frames_red.mkv"
        with common.cloned_sample(sample_name):
            self.check_import([sample_name])

            project = self.app.project_manager.current_project
            timeline_container = TimelineContainer(self.app, editor_state=self.app.gui.editor.editor_state)
            timeline_container.set_project(project)

            assets = project.list_assets(GES.UriClip)
            asset, = [a for a in assets if "proxy" in a.props.id]
            layer, = project.ges_timeline.get_layers()
            clip = asset.extract()
            layer.add_clip(clip)
            video_source = clip.find_track_element(None, GES.VideoUriSource)
            self.assertEqual(video_source.get_child_property("width")[1], 320)
            self.assertEqual(video_source.get_child_property("height")[1], 240)

            dialog = self.create_rendering_dialog(project)

            # Simulate setting the scale to 10%.
            with mock.patch.object(dialog.scale_spinbutton, "get_value",
                                   return_value=10):
                dialog._scale_spinbutton_changed_cb(None)
                self.render(dialog)

            self.mainloop.run(until_empty=True)

            video_source = clip.find_track_element(None, GES.VideoUriSource)
            self.assertEqual(video_source.get_child_property("width")[1], 320)
            self.assertEqual(video_source.get_child_property("height")[1], 240)

    # pylint: disable=invalid-name
    def test_rendering_with_scaled_proxies(self):
        """Tests rendering with scaled proxies."""
        sample_name = "30fps_numeroted_frames_red.mkv"
        with common.cloned_sample(sample_name):
            self.check_import([sample_name], proxying_strategy=ProxyingStrategy.NOTHING)

            project = self.app.project_manager.current_project
            proxy_manager = self.app.proxy_manager
            timeline_container = TimelineContainer(self.app, editor_state=self.app.gui.editor.editor_state)
            timeline_container.set_project(project)
            rendering_asset = None

            asset, = project.list_assets(GES.UriClip)
            proxy = self.check_add_proxy(asset, scaled=True)

            layer, = project.ges_timeline.get_layers()
            clip = proxy.extract()
            layer.add_clip(clip)

            # Patch the function that reverts assets to proxies after rendering.
            from pitivi.render import RenderDialog
            old_use_proxy_assets = RenderDialog._use_proxy_assets

            def check_use_proxy_assets(self):
                nonlocal layer, asset, rendering_asset
                clip, = layer.get_clips()
                rendering_asset = clip.get_asset()
                old_use_proxy_assets(self)

            RenderDialog._use_proxy_assets = check_use_proxy_assets
            try:
                dialog = self.create_rendering_dialog(project)
                self.render(dialog)
                self.mainloop.run(until_empty=True)
            finally:
                RenderDialog._use_proxy_assets = old_use_proxy_assets

            # Check rendering did not use scaled proxy
            self.assertFalse(proxy_manager.is_scaled_proxy(rendering_asset))
            # Check asset was replaced with scaled proxy after rendering
            self.assertTrue(proxy_manager.is_scaled_proxy(clip.get_asset()))

    # pylint: disable=invalid-name
    def test_rendering_with_unsupported_asset_scaled_proxies(self):
        """Tests rendering with scaled proxies."""
        sample_name = "30fps_numeroted_frames_red.mkv"
        with common.cloned_sample(sample_name):
            self.check_import([sample_name], proxying_strategy=ProxyingStrategy.AUTOMATIC)

            project = self.app.project_manager.current_project
            proxy_manager = self.app.proxy_manager
            timeline_container = TimelineContainer(self.app, editor_state=self.app.gui.editor.editor_state)
            timeline_container.set_project(project)
            rendering_asset = None

            asset, = project.list_assets(GES.UriClip)
            with mock.patch.object(proxy_manager,
                                   "is_asset_format_well_supported",
                                   return_value=False):
                proxy = self.check_add_proxy(asset, scaled=True)

                # Check that HQ proxy was created
                hq_uri = self.app.proxy_manager.get_proxy_uri(asset)
                self.assertTrue(os.path.exists(Gst.uri_get_location(hq_uri)), hq_uri)

                layer, = project.ges_timeline.get_layers()
                clip = proxy.extract()
                layer.add_clip(clip)

                def _use_proxy_assets():
                    nonlocal layer, asset, rendering_asset
                    clip, = layer.get_clips()
                    rendering_asset = clip.get_asset()
                    old_use_proxy_assets()

                dialog = self.create_rendering_dialog(project)
                old_use_proxy_assets = dialog._use_proxy_assets
                dialog._use_proxy_assets = _use_proxy_assets
                self.render(dialog)
                self.mainloop.run(until_empty=True)

                # Check rendering used HQ proxy
                self.assertTrue(proxy_manager.is_hq_proxy(rendering_asset))
                # Check asset was replaced with scaled proxy after rendering
                self.assertTrue(proxy_manager.is_scaled_proxy(clip.get_asset()))

    @skipUnless(*encoding_target_exists("youtube"))
    # pylint: disable=invalid-name
    def test_rendering_with_youtube_profile(self):
        """Tests rendering a simple timeline with the youtube profile."""
        self.check_simple_rendering_profile("youtube")

    @skipUnless(*encoding_target_exists("youtube"))
    def test_preset_reset_when_changing_muxer(self):
        """Tests setting the container profile manually."""
        project = self.create_simple_project()
        dialog = self.create_rendering_dialog(project)
        self.select_render_preset(dialog, "youtube")

        # The container and video encoder profiles in the "youtube"
        # EncodingTarget are "qtmux" and "x264enc". They have a common
        # "preset" called "Profile YouTube".
        # When changing the container manually from qt4mux to mp4mux
        # the container profile's "preset" needs to be reset, otherwise
        # rendering will hang because mp4mux is missing the
        # "Profile YouTube" preset.
        self.assertTrue(set_combo_value(dialog.muxer_combo,
                                        Gst.ElementFactory.find("mp4mux")))

        self.render(dialog)

    def test_preset_changes_file_extension(self):
        """Test file extension changes according to the chosen preset."""
        project = self.create_simple_project()
        dialog = self.create_rendering_dialog(project)
        self.select_render_preset(dialog, "youtube")
        self.assertTrue(dialog.fileentry.get_text().endswith("mov"))

        self.select_render_preset(dialog, "dvd")
        self.assertTrue(dialog.fileentry.get_text().endswith("mpeg"))

        self.select_render_preset(dialog, "youtube")
        self.assertTrue(dialog.fileentry.get_text().endswith("mov"))

    def select_render_preset(self, dialog, profile_name):
        """Sets the preset value for an existing dialog."""
        row = None
        for item in dialog.presets_manager.model:
            if item.name == profile_name:
                row = mock.Mock()
                row.preset_item = item
                break
        self.assertIsNotNone(row)

        dialog._preset_listbox_row_activated_cb(None, row)
        self.assertEqual(dialog.presets_manager.cur_preset_item.name, profile_name)

    @skipUnless(*encoding_target_exists("dvd"))
    def test_rendering_with_dvd_profile(self):
        """Tests rendering a simple timeline with the DVD profile."""
        self.check_simple_rendering_profile("dvd")

    def test_rendering_with_default_profile(self):
        """Tests rendering a simple timeline with the default profile."""
        self.check_simple_rendering_profile()

    @skipUnless(*encoding_target_exists("youtube"))
    def test_setting_caps_fields_in_advanced_dialog(self):
        """Tests setting special advanced setting (which are actually set on caps)."""
        project = self.create_simple_project()
        dialog = self.create_rendering_dialog(project)
        self.select_render_preset(dialog, "youtube")

        dialog.window = None  # Make sure the dialog window is never set to Mock.
        dialog._video_settings_button_clicked_cb(None)
        self.assertEqual(dialog.dialog.elementsettings.get_caps_values(), {"profile": "high"})

        dialog.dialog.elementsettings.caps_widgets["profile"].set_widget_value("baseline")
        self.assertEqual(dialog.dialog.elementsettings.get_caps_values(), {"profile": "baseline"})

        caps = dialog.dialog.get_caps()
        self.assert_caps_equal(caps, "video/x-h264,profile=baseline")

        dialog.dialog.ok_btn.emit("clicked")
        self.assert_caps_equal(project.video_profile.get_format(), "video/x-h264,profile=baseline")

        dialog._video_settings_button_clicked_cb(None)

        caps = dialog.dialog.get_caps()
        self.assert_caps_equal(caps, "video/x-h264,profile=baseline")

    def check_quality_widget(self, dialog, vencoder, vcodecsettings, preset, sensitive, value):
        if vencoder:
            self.assertEqual(dialog.project.vencoder, vencoder)
        if vcodecsettings is not None:
            self.assertDictEqual(dialog.project.vcodecsettings, vcodecsettings)

        if preset:
            self.assertEqual(dialog.presets_manager.cur_preset_item.name, preset)
        else:
            self.assertIsNone(dialog.presets_manager.cur_preset_item)

        self.assertEqual(dialog.quality_scale.props.sensitive, sensitive)
        self.assertEqual(dialog.quality_adjustment.props.value, value)

    @skipUnless(*encoding_target_exists("dvd"))
    @skipUnless(*encoding_target_exists("youtube"))
    @skipUnless(*factory_exists("pngenc"))
    def test_quality_widget(self):
        project = self.create_simple_project()
        dialog = self.create_rendering_dialog(project)
        self.check_quality_widget(dialog,
                                  vencoder="x264enc", vcodecsettings={"quantizer": 21, "pass": 5},
                                  preset="youtube",
                                  sensitive=True, value=Quality.MEDIUM)

        self.assertEqual(project.video_profile.get_preset_name(), "x264enc")
        dialog.quality_adjustment.props.value = Quality.HIGH
        self.check_quality_widget(dialog,
                                  vencoder="x264enc", vcodecsettings={"quantizer": 18, "pass": 5},
                                  preset="youtube",
                                  sensitive=True, value=Quality.HIGH)

        self.select_render_preset(dialog, "dvd")
        self.check_quality_widget(dialog,
                                  vencoder=None, vcodecsettings={},
                                  preset="dvd",
                                  sensitive=False, value=Quality.LOW)

        self.select_render_preset(dialog, "youtube")
        self.assertEqual(project.video_profile.get_preset_name(), "x264enc")
        self.check_quality_widget(dialog,
                                  vencoder="x264enc", vcodecsettings={"quantizer": 21, "pass": 5},
                                  preset="youtube",
                                  sensitive=True, value=Quality.MEDIUM)

        self.assertTrue(set_combo_value(dialog.video_encoder_combo,
                                        Gst.ElementFactory.find("pngenc")))
        self.check_quality_widget(dialog,
                                  vencoder="pngenc", vcodecsettings={},
                                  preset=None,
                                  sensitive=False, value=Quality.LOW)

        self.select_render_preset(dialog, "youtube")
        self.check_quality_widget(dialog,
                                  vencoder="x264enc", vcodecsettings={"quantizer": 21, "pass": 5},
                                  preset="youtube",
                                  sensitive=True, value=Quality.MEDIUM)

    def test_preset_persistent(self):
        """Checks the render preset is remembered when loading a project."""
        project = self.create_simple_project()
        self.assertEqual(project.muxer, "webmmux")
        self.assertEqual(project.vencoder, "vp8enc")
        self.assertDictEqual(project.vcodecsettings, {})

        dialog = self.create_rendering_dialog(project)
        self.check_quality_widget(dialog,
                                  vencoder="x264enc", vcodecsettings={"quantizer": 21, "pass": 5},
                                  preset="youtube",
                                  sensitive=True, value=Quality.MEDIUM)

        project_manager = project.app.project_manager
        with tempfile.NamedTemporaryFile() as temp_file:
            uri = Gst.filename_to_uri(temp_file.name)
            project_manager.save_project(uri=uri, backup=False)

            app2 = common.create_pitivi()
            project2 = app2.project_manager.load_project(uri)
            timeline_container = TimelineContainer(app2, editor_state=app2.gui.editor.editor_state)
            timeline_container.set_project(project2)
            common.create_main_loop().run(until_empty=True)
            self.assertEqual(project2.muxer, "qtmux")
            self.assertEqual(project2.vencoder, "x264enc")
            self.assertTrue(set({"quantizer": 21, "pass": 5}.items()).issubset(set(project2.vcodecsettings.items())))

        dialog2 = self.create_rendering_dialog(project2)
        self.assertTrue(set({"quantizer": 21, "pass": 5}.items()).issubset(set(project2.vcodecsettings.items())))
        self.check_quality_widget(dialog2,
                                  vencoder="x264enc", vcodecsettings=None,
                                  preset="youtube",
                                  sensitive=True, value=Quality.MEDIUM)

    def test_project_audiorate(self):
        """Checks the project audiorate when opening the Render dialog."""
        project = self.create_simple_project()
        # This is the audiorate from tears_of_steel.webm.
        self.assertEqual(project.audiorate, 44100)

        unused_dialog = self.create_rendering_dialog(project)

        # The audio rate is changed because the default render preset
        # has an audio encoder which does not support 44100.
        self.assertEqual(project.audiorate, 48000)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2014, Alex Băluț <alexandru.balut@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the pitivi.clipproperties module."""
# pylint: disable=protected-access,no-self-use,import-outside-toplevel,no-member
import tempfile
from unittest import mock

from gi.repository import GES
from gi.repository import Gst

from tests import common


class TransformationPropertiesTest(common.TestCase):
    """Tests for the TransformationProperties widget."""

    @common.setup_timeline
    @common.setup_clipproperties
    def test_spin_buttons_read(self):
        """Checks the spin buttons update when the source properties change."""
        timeline = self.transformation_box.app.gui.editor.timeline_ui.timeline
        spin_buttons = self.transformation_box.spin_buttons

        # Add a clip and select it
        clip = self.add_clips_simple(timeline, 1)[0]
        timeline.selection.select([clip])

        # Check that spin buttons display the correct values by default
        source = self.transformation_box.source
        self.assertIsNotNone(source)
        for prop in ["posx", "posy", "width", "height"]:
            self.assertIn(prop, spin_buttons)
            ret, source_value = source.get_child_property(prop)
            self.assertTrue(ret)
            spin_btn_value = spin_buttons[prop].get_value_as_int()
            self.assertEqual(spin_btn_value, source_value)

        # Change the source properties and check the spin buttons update
        # correctly.
        new_values = {"posx": 20, "posy": -50, "width": 70, "height": 450}
        for prop, new_val in new_values.items():
            self.assertTrue(source.set_child_property(prop, new_val))
            spin_btn_value = spin_buttons[prop].get_value_as_int()
            self.assertEqual(new_val, spin_btn_value)

    @common.setup_timeline
    @common.setup_clipproperties
    def test_spin_buttons_write(self):
        """Checks the spin buttons changing updates the source properties."""
        timeline = self.transformation_box.app.gui.editor.timeline_ui.timeline
        spin_buttons = self.transformation_box.spin_buttons

        # Add a clip and select it
        clip = self.add_clips_simple(timeline, 1)[0]
        timeline.selection.select([clip])
        source = self.transformation_box.source
        self.assertIsNotNone(source)

        # Get current spin buttons values
        current_spin_values = {}
        for prop in ["posx", "posy", "width", "height"]:
            current_spin_values[prop] = spin_buttons[prop].get_value_as_int()

        changes = [
            ("posx", -300), ("posy", 450), ("width", 1), ("height", 320),
            ("posx", 230), ("posx", 520), ("posy", -10), ("posy", -1000),
            ("width", 600), ("width", 1000), ("height", 1), ("height", 1000)
        ]

        # Change the spin buttons values and check the source properties are
        # updated correctly.
        for prop, new_value in changes:
            spin_buttons[prop].set_value(new_value)
            current_spin_values[prop] = new_value
            for source_prop in ["posx", "posy", "width", "height"]:
                ret, source_value = source.get_child_property(source_prop)
                self.assertTrue(ret)
                self.assertEqual(current_spin_values[source_prop], source_value)

    @common.setup_timeline
    @common.setup_clipproperties
    def test_spin_buttons_source_change(self):
        """Checks the spin buttons update when the selected clip changes."""
        timeline = self.transformation_box.app.gui.editor.timeline_ui.timeline
        spin_buttons = self.transformation_box.spin_buttons

        # Add two clips and select the first one
        clips = self.add_clips_simple(timeline, 2)
        timeline.selection.select([clips[0]])
        source = self.transformation_box.source
        self.assertIsNotNone(source)

        # Change the spin buttons values
        new_values = {"posx": 45, "posy": 10, "width": 450, "height": 25}
        for prop, new_val in new_values.items():
            spin_buttons[prop].set_value(new_val)

        # Select the second clip and check the spin buttons values update
        # correctly
        timeline.selection.select([clips[1]])
        source = self.transformation_box.source
        self.assertIsNotNone(source)
        for prop in ["posx", "posy", "width", "height"]:
            ret, source_value = source.get_child_property(prop)
            self.assertTrue(ret)
            self.assertEqual(spin_buttons[prop].get_value_as_int(), source_value)

        # Select the first clip again and check spin buttons values
        timeline.selection.select([clips[0]])
        for prop in ["posx", "posy", "width", "height"]:
            self.assertEqual(spin_buttons[prop].get_value_as_int(), new_values[prop])

    @common.setup_timeline
    @common.setup_clipproperties
    def test_keyframes_activate(self):
        """Checks transformation properties keyframes activation."""
        timeline = self.transformation_box.app.gui.editor.timeline_ui.timeline

        # Add a clip and select it
        clip = self.add_clips_simple(timeline, 1)[0]
        timeline.selection.select([clip])
        source = self.transformation_box.source
        self.assertIsNotNone(source)
        inpoint = source.props.in_point
        duration = source.props.duration

        # Check keyframes are deactivated by default
        for prop in ["posx", "posy", "width", "height"]:
            self.assertIsNone(source.get_control_binding(prop))

        # Get current source properties
        initial_values = {}
        for prop in ["posx", "posy", "width", "height"]:
            ret, value = source.get_child_property(prop)
            self.assertTrue(ret)
            initial_values[prop] = value

        # Activate keyframes and check the default keyframes are created
        self.transformation_box._activate_keyframes_btn.set_active(True)
        for prop in ["posx", "posy", "width", "height"]:
            control_binding = source.get_control_binding(prop)
            self.assertIsNotNone(control_binding)
            control_source = control_binding.props.control_source
            keyframes = [(item.timestamp, item.value) for item in control_source.get_all()]
            self.assertEqual(keyframes, [(inpoint, initial_values[prop]),
                                         (inpoint + duration, initial_values[prop])])

    @common.setup_timeline
    @common.setup_clipproperties
    def test_keyframes_add(self):
        """Checks keyframe creation."""
        timeline = self.transformation_box.app.gui.editor.timeline_ui.timeline
        pipeline = timeline._project.pipeline
        spin_buttons = self.transformation_box.spin_buttons

        # Add a clip and select it
        clip = self.add_clips_simple(timeline, 1)[0]
        timeline.selection.select([clip])
        source = self.transformation_box.source
        self.assertIsNotNone(source)
        start = source.props.start
        inpoint = source.props.in_point
        duration = source.props.duration

        # Activate keyframes
        self.transformation_box._activate_keyframes_btn.set_active(True)

        # Add some more keyframes
        offsets = [1, int(duration / 2), duration - 1]
        for prop in ["posx", "posy", "width", "height"]:
            for index, offset in enumerate(offsets):
                timestamp, value = inpoint + offset, offset * 10
                with mock.patch.object(pipeline, "get_position") as get_position:
                    get_position.return_value = start + offset
                    spin_buttons[prop].set_value(value)

                control_source = source.get_control_binding(prop).props.control_source
                keyframes = [(item.timestamp, item.value) for item in control_source.get_all()]
                self.assertEqual((timestamp, value), keyframes[index + 1])

    @common.setup_timeline
    @common.setup_clipproperties
    def test_keyframes_navigation(self):
        """Checks keyframe navigation."""
        timeline = self.transformation_box.app.gui.editor.timeline_ui.timeline
        pipeline = timeline._project.pipeline

        # Add a clip and select it
        clip = self.add_clips_simple(timeline, 1)[0]
        timeline.selection.select([clip])
        source = self.transformation_box.source
        self.assertIsNotNone(source)
        start = source.props.start
        inpoint = source.props.in_point
        duration = source.props.duration

        # Activate keyframes and add some more keyframes
        self.transformation_box._activate_keyframes_btn.set_active(True)
        offsets = [1, int(duration / 2), duration - 1]
        for prop in ["posx", "posy", "width", "height"]:
            for offset in offsets:
                timestamp, value = inpoint + offset, offset * 10
                control_source = source.get_control_binding(prop).props.control_source
                control_source.set(timestamp, value)

        # Add edge keyframes in the offsets array
        offsets.insert(0, 0)
        offsets.append(duration)

        # Test keyframe navigation
        prev_index = 0
        next_index = 1
        for position in range(duration + 1):
            prev_keyframe_ts = offsets[prev_index] + inpoint
            next_keyframe_ts = offsets[next_index] + inpoint

            with mock.patch.object(pipeline, "get_position") as get_position:
                get_position.return_value = start + position
                with mock.patch.object(pipeline, "simple_seek") as simple_seek:
                    self.transformation_box._prev_keyframe_btn.clicked()
                    simple_seek.assert_called_with(prev_keyframe_ts)
                    self.transformation_box._next_keyframe_btn.clicked()
                    simple_seek.assert_called_with(next_keyframe_ts)

            if position + 1 == next_keyframe_ts and next_index + 1 < len(offsets):
                next_index += 1
            if position in offsets and position != 0:
                prev_index += 1

    @common.setup_timeline
    @common.setup_clipproperties
    def test_reset_to_default(self):
        """Checks "reset to default" button."""
        timeline = self.transformation_box.app.gui.editor.timeline_ui.timeline

        # Add a clip and select it
        clip = self.add_clips_simple(timeline, 1)[0]
        timeline.selection.select([clip])
        source = self.transformation_box.source
        self.assertIsNotNone(source)

        # Change source properties
        new_values = {"posx": 20, "posy": -50, "width": 70, "height": 450}
        for prop, new_val in new_values.items():
            self.assertTrue(source.set_child_property(prop, new_val))

        # Activate keyframes
        self.transformation_box._activate_keyframes_btn.set_active(True)

        # Press "reset to default" button
        clear_button = self.transformation_box.builder.get_object("clear_button")
        clear_button.clicked()

        # Check that control bindings were erased and the properties were
        # reset to their default values
        for prop in ["posx", "posy", "width", "height"]:
            self.assertIsNone(source.get_control_binding(prop))
            ret, value = source.get_child_property(prop)
            self.assertTrue(ret)
            self.assertEqual(value, source.ui.default_position[prop])


class TitlePropertiesTest(common.TestCase):
    """Tests for the TitleProperties class."""

    def _get_title_source_child_props(self):
        clips = self.layer.get_clips()
        self.assertEqual(len(clips), 1, clips)
        self.assertIsInstance(clips[0], GES.TitleClip)
        source, = clips[0].get_children(False)
        return {p: source.get_child_property(p)
                for p in ("text",
                          "x-absolute", "y-absolute",
                          "valignment", "halignment",
                          "font-desc",
                          "color",
                          "foreground-color")}

    @common.setup_timeline
    @common.setup_clipproperties
    def test_create_title(self):
        """Exercise creating a title clip."""
        self.project.pipeline.get_position = mock.Mock(return_value=0)

        self.clipproperties.create_title_clip_cb(None)
        ps1 = self._get_title_source_child_props()
        self.assertTrue(ps1["text"][1])
        self.assertNotEqual(ps1["text"][1], "", "Title clip does not have an initial text")

        self.action_log.undo()
        clips = self.layer.get_clips()
        self.assertEqual(len(clips), 0, clips)

        self.action_log.redo()
        ps2 = self._get_title_source_child_props()
        self.assertDictEqual(ps1, ps2)

    @common.setup_timeline
    @common.setup_clipproperties
    def test_modify_title(self):
        """Exercise modifying the title."""
        self.project.pipeline.get_position = mock.Mock(return_value=0)

        self.clipproperties.create_title_clip_cb(None)
        ps1 = self._get_title_source_child_props()

        # Modify the title.
        mod_title = "Modifed Title"
        self.clipproperties.title_expander.textbuffer.props.text = mod_title
        ps2 = self._get_title_source_child_props()
        self.assertEqual(ps2["text"][1], mod_title)
        self.assertNotEqual(ps1["text"], ps2["text"])

        # Undo modify title.
        self.action_log.undo()
        ps3 = self._get_title_source_child_props()
        self.assertDictEqual(ps1, ps3)

        # Redo modify title.
        self.action_log.redo()
        ps4 = self._get_title_source_child_props()
        self.assertDictEqual(ps2, ps4)

    @common.setup_timeline
    @common.setup_clipproperties
    def test_selection_does_nothing(self):
        """Checks de/selection do not create undoable operations."""
        self.project.pipeline.get_position = mock.Mock(return_value=0)
        self.clipproperties.create_title_clip_cb(None)
        self.assertEqual(len(self.action_log.undo_stacks), 1)
        clips = self.layer.get_clips()
        self.assertEqual(len(clips), 1, clips)

        self.timeline_container.timeline.selection.unselect(clips)
        self.assertEqual(len(self.action_log.undo_stacks), 1)

        self.timeline_container.timeline.selection.select(clips)
        self.assertEqual(len(self.action_log.undo_stacks), 1)

    @common.setup_timeline
    @common.setup_clipproperties
    def test_xxx(self):
        """Exercise creating a title clip."""
        self.project.pipeline.get_position = mock.Mock(return_value=0)

        # Create the first clip.
        self.clipproperties.create_title_clip_cb(None)
        clip1, = self.layer.get_clips()
        source1, = clip1.get_children(False)
        self.clipproperties.title_expander.textbuffer.props.text = "TC1"
        self.assertEqual(source1.get_child_property("text"), (True, "TC1"))

        # Make place for the second clip at the beginning of the layer.
        clip1.props.start = clip1.props.duration

        # Create the second clip.
        self.clipproperties.create_title_clip_cb(None)
        clip2, clip1_ = self.layer.get_clips()
        self.assertIs(clip1_, clip1)
        source2, = clip2.get_children(False)
        self.clipproperties.title_expander.textbuffer.props.text = "TC2"
        self.assertEqual(source2.get_child_property("text"), (True, "TC2"))

        self.assertEqual(source2.get_child_property("text"), (True, "TC2"))
        self.assertEqual(source1.get_child_property("text"), (True, "TC1"))

        # Switch back to clip1.
        self.timeline_container.timeline.selection.select([clip1])
        self.assertEqual(source1.get_child_property("text"), (True, "TC1"))
        self.assertEqual(source2.get_child_property("text"), (True, "TC2"))

        # Switch back to clip2.
        self.timeline_container.timeline.selection.select([clip2])
        self.assertEqual(source1.get_child_property("text"), (True, "TC1"))
        self.assertEqual(source2.get_child_property("text"), (True, "TC2"))


class ClipPropertiesTest(common.TestCase):
    """Tests for the ClipProperties class."""

    @common.setup_timeline
    @common.setup_clipproperties
    def test_alignment_editor(self):
        """Exercise aligning a clip using the alignment editor."""
        self.project.pipeline.get_position = mock.Mock(return_value=0)

        timeline = self.timeline_container.timeline
        clip = self.add_clips_simple(timeline, 1)[0]
        timeline.selection.select([clip])
        source = self.transformation_box.source
        self.assertIsNotNone(source)

        height = source.get_child_property("height").value
        width = source.get_child_property("width").value

        self.assertEqual(source.get_child_property("posx").value, 0)
        self.assertEqual(source.get_child_property("posy").value, 0)

        alignment_editor = self.transformation_box.alignment_editor
        event = mock.MagicMock()
        event.x = 0
        event.y = 0
        alignment_editor._motion_notify_event_cb(None, event)
        alignment_editor._button_release_event_cb(None, None)

        self.assertEqual(source.get_child_property("posx").value, -width)
        self.assertEqual(source.get_child_property("posy").value, -height)

        self.action_log.undo()

        self.assertEqual(source.get_child_property("posx").value, 0)
        self.assertEqual(source.get_child_property("posy").value, 0)

        self.action_log.redo()

        self.assertEqual(source.get_child_property("posx").value, -width)
        self.assertEqual(source.get_child_property("posy").value, -height)


class SpeedPropertiesTest(common.TestCase):
    """Tests for the TransformationProperties widget."""

    def assert_applied_rate(self, sources_count, rate, duration):
        self.assertEqual(len(self.speed_box._time_effects), sources_count)
        self.assertEqual(self.speed_box.props.rate, rate)
        self.assertEqual(self.speed_box._clip.props.duration, duration)
        for effect, propname in self.speed_box._time_effects.values():
            self.assertTrue(propname in ["rate", "tempo"], propname)
            self.assertEqual(effect.get_child_property(propname).value, rate)

        self.assertEqual(self.speed_box._speed_adjustment.props.value, rate)

    def assert_clip_speed_child_props(self, clip, audio, video, value):
        if audio:
            self.assertEqual(clip.get_child_property("tempo").value, value)
        if video:
            self.assertEqual(clip.get_child_property("GstVideoRate::rate").value, value)

    def _check_clip_speed(self, audio=False, video=False):
        sources_count = len([source for source in [audio, video] if source])

        clip, = self.layer.get_clips()

        duration = self.project.ges_timeline.get_frame_time(self.project.ges_timeline.get_frame_at(Gst.SECOND))
        self.project.ges_timeline.props.snapping_distance = duration
        self.assertEqual(self.speed_box._sources, {})
        self.assertEqual(self.speed_box._time_effects, {})

        self.timeline_container.timeline.selection.select([clip])

        self.assertEqual(len(self.speed_box._sources), sources_count, self.speed_box._sources)
        self.assertEqual(self.speed_box._time_effects, {})

        clip.props.duration = duration
        self.assertEqual(self.speed_box._clip.props.duration, duration)

        self.speed_box._speed_adjustment.props.value = 2.0
        self.assert_applied_rate(sources_count, 2.0, int(duration / 2))

        self.speed_box._speed_adjustment.props.value = 0.5
        self.assert_applied_rate(sources_count, 0.5, int(duration / 2) * 4)

        self.action_log.undo()
        self.assert_applied_rate(sources_count, 2.0, int(duration / 2))

        self.action_log.undo()
        self.assert_applied_rate(sources_count, 1.0, duration)

        self.action_log.redo()
        self.assert_applied_rate(sources_count, 2.0, int(duration / 2))

        self.action_log.redo()
        self.assert_applied_rate(sources_count, 0.5, int(duration / 2) * 4)

        self.timeline_container.timeline.selection.select([])
        self.assertEqual(self.speed_box._sources, {})
        self.assertEqual(self.speed_box._time_effects, {})

        self.timeline_container.timeline.selection.select([clip])
        self.assert_applied_rate(sources_count, 0.5, int(duration / 2) * 4)

        self.action_log.undo()
        self.assert_applied_rate(sources_count, 2.0, int(duration / 2))

        self.timeline_container.timeline.selection.select([])
        self.assertEqual(self.speed_box._sources, {})
        self.assertEqual(self.speed_box._time_effects, {})

        self.action_log.undo()
        self.assert_clip_speed_child_props(clip, audio, video, 1.0)

        self.action_log.redo()
        self.assert_clip_speed_child_props(clip, audio, video, 2.0)

        self.action_log.redo()
        self.assert_clip_speed_child_props(clip, audio, video, 0.5)

        self.timeline_container.timeline.selection.select([clip])
        total_duration = clip.props.duration
        self.project.pipeline.get_position = mock.Mock(return_value=duration)
        self.timeline_container.split_action.emit("activate", None)

        clip1, clip2 = self.layer.get_clips()
        self.assertEqual(clip1.props.start, 0)
        self.assertEqual(clip1.props.duration, duration)
        self.assertEqual(clip2.props.start, duration)
        self.assertEqual(clip2.props.duration, total_duration - duration)
        self.assertEqual(self.project.ges_timeline.props.snapping_distance, duration)

        # 0.1 would lead to clip1 totally overlapping clip2, ensure it is a noop
        self.speed_box._speed_adjustment.props.value = 0.1
        self.assert_applied_rate(sources_count, 0.5, duration)
        self.assertEqual(self.project.ges_timeline.props.snapping_distance, duration)

        self.action_log.undo()
        self.assert_applied_rate(sources_count, 0.5, int(duration / 2) * 4)

        # Undoing should undo the split
        clip1, = self.layer.get_clips()

        # redo the split
        self.action_log.redo()
        clip1, clip2 = self.layer.get_clips()
        self.assertEqual(self.speed_box._clip, clip1)
        self.assert_applied_rate(sources_count, 0.5, duration)
        self.assertEqual(self.project.ges_timeline.props.snapping_distance, duration)

        self.speed_box._speed_adjustment.props.value = 1.0
        self.assert_applied_rate(sources_count, 1.0, int(duration / 2))

        self.speed_box._speed_adjustment.props.value = 0.5
        self.assert_applied_rate(sources_count, 0.5, int(duration / 2) * 2)

        self.speed_box.set_clip(None)
        self.assertEqual(self.speed_box._sources, {})
        self.assertEqual(self.speed_box._time_effects, {})

    @common.setup_project_with_clips(assets_names=["1sec_simpsons_trailer.mp4"])
    @common.setup_clipproperties
    def test_clip_speed_av(self):
        self._check_clip_speed(audio=True, video=True)

    @common.setup_project_with_clips(assets_names=["mp3_sample.mp3"])
    @common.setup_clipproperties
    def test_clip_speed_a(self):
        self._check_clip_speed(audio=True)

    @common.setup_project_with_clips(assets_names=["30fps_numeroted_frames_blue.webm"])
    @common.setup_clipproperties
    def test_clip_speed_v(self):
        self._check_clip_speed(video=True)

    @common.setup_project_with_clips(assets_names=["mp3_sample.mp3", "30fps_numeroted_frames_blue.webm"])
    @common.setup_clipproperties
    def test_widgets_updated_when_switching_clips(self):
        clip1, clip2 = self.layer.get_clips()
        clip1_duration = clip1.props.duration
        clip2_duration = clip2.props.duration

        self.timeline_container.timeline.selection.select([clip1])
        self.assertIs(self.speed_box._clip, clip1)
        self.assert_applied_rate(0, 1.0, clip1_duration)

        self.speed_box._speed_adjustment.props.value = 2.0
        self.assert_applied_rate(1, 2.0, clip1_duration / 2)

        self.timeline_container.timeline.selection.select([clip2])
        self.assertIs(self.speed_box._clip, clip2)
        self.assert_applied_rate(0, 1.0, clip2_duration)

        self.timeline_container.timeline.selection.select([clip1])
        self.assert_applied_rate(1, 2.0, clip1_duration / 2)

    @common.setup_project_with_clips(assets_names=["1sec_simpsons_trailer.mp4"])
    @common.setup_clipproperties
    def test_load_project_clip_speed(self):
        sources_count = 2
        clip, = self.layer.get_clips()
        clip.props.duration = Gst.SECOND

        self.timeline_container.timeline.selection.select([clip])
        self.speed_box._speed_adjustment.props.value = 0.5
        self.assert_applied_rate(sources_count, 0.5, 2 * Gst.SECOND)

        with tempfile.NamedTemporaryFile() as temp_file:
            uri = Gst.filename_to_uri(temp_file.name)
            project_manager = self.app.project_manager
            self.assertTrue(project_manager.save_project(uri=uri, backup=False))

            mainloop = common.create_main_loop()

            project_manager.connect("new-project-loaded", lambda *args: mainloop.quit())
            project_manager.connect("closing-project", lambda *args: True)
            self.assertTrue(project_manager.close_running_project())
            project_manager.load_project(uri)
            mainloop.run()

        new_clip, = self.layer.get_clips()
        self.assertNotEqual(new_clip, clip)

        self.timeline_container.timeline.selection.select([new_clip])
        self.assert_applied_rate(sources_count, 0.5, 2 * Gst.SECOND)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2020, Caleb Marcoux <caleb.marcoux@gmail.com> (primary contact),
# Copyright (c) 2020, Abby Brooks <abigail.brooks@huskers.unl.edu>,
# Copyright (c) 2020, Won Choi <won.choi@huskers.unl.edu>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the pitivi.editorstate module."""
import tempfile
from unittest import mock

from pitivi.editorstate import EditorState
from tests import common


class TestEditorState(common.TestCase):
    """Tests the EditorState class."""

    def test_save_load(self):
        values = {
            "playhead-position": 5000000000,
            "zoom-level": 50,
            "scroll": 50.0,
            "selection": []
        }
        saved = EditorState(mock.Mock())
        saved.conf_file_path = tempfile.NamedTemporaryFile().name
        for key, value in values.items():
            saved.set_value(key, value)
        saved.save_editor_state()

        loaded = EditorState(mock.Mock())
        loaded.conf_file_path = saved.conf_file_path
        loaded.load_editor_state()
        for key, value in values.items():
            self.assertEqual(loaded.get_value(key), value)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2015, Thibault Saunier <tsaunier@gnome.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the pitivi.medialibrary module."""
# pylint: disable=attribute-defined-outside-init,protected-access
import os
import tempfile
from unittest import mock

from gi.repository import Gdk
from gi.repository import GES
from gi.repository import Gst

from pitivi.medialibrary import AssetThumbnail
from pitivi.medialibrary import MediaLibraryWidget
from pitivi.medialibrary import TagState
from pitivi.medialibrary import ViewType
from pitivi.project import ProjectManager
from pitivi.utils.misc import ASSET_DURATION_META
from pitivi.utils.misc import asset_get_duration
from pitivi.utils.proxy import ProxyingStrategy
from pitivi.utils.validate import create_event
from tests import common


class BaseTestMediaLibrary(common.TestCase):

    def __init__(self, *args):
        common.TestCase.__init__(self, *args)
        self.app = None
        self.medialibrary = None
        self.mainloop = None

    def tearDown(self):
        self.clean()
        common.TestCase.tearDown(self)

    def clean(self):
        self.mainloop = None
        self.app = None

        if self.medialibrary:
            self.medialibrary.finalize()
            self.medialibrary = None

    def _custom_set_up(self, project_uri=None, **settings):
        # Always make sure we start with a clean medialibrary, and no other
        # is connected to some assets.
        self.clean()

        self.mainloop = common.create_main_loop()
        self.check_no_transcoding = False
        self.app = common.create_pitivi_mock(**settings)
        self.app.project_manager = ProjectManager(self.app)
        self.medialibrary = MediaLibraryWidget(self.app)

        if project_uri:
            self.app.project_manager.load_project(project_uri)
        else:
            self.app.project_manager.new_blank_project()

        self.app.project_manager.current_project.connect(
            "loaded", self.project_loaded_cb)
        self.mainloop.run()

    def project_loaded_cb(self, unused_project, unused_timeline):
        self.mainloop.quit()

    def _progress_bar_cb(self, progressbar, unused_pspec):
        if self.check_no_transcoding:
            self.assertTrue(progressbar.props.fraction == 1.0 or
                            progressbar.props.fraction == 0.0,
                            "Some transcoding is happening, got progress: %f"
                            % progressbar.props.fraction)

        if progressbar.props.fraction == 1.0:
            self.assertEqual(self.medialibrary.store.get_n_items(),
                             len(self.samples))
            self.mainloop.quit()

    def _create_assets(self, samples):
        self.samples = samples
        for sample_name in samples:
            self.app.project_manager.current_project.create_asset(
                common.get_sample_uri(sample_name), GES.UriClip)

    def check_import(self, samples, proxying_strategy=ProxyingStrategy.ALL,
                     check_no_transcoding=False, auto_scaling_enabled=False):
        """Simulates the user importing an asset."""
        self._custom_set_up(proxying_strategy=proxying_strategy,
                            num_transcoding_jobs=4,
                            last_clip_view=ViewType.LIST,
                            auto_scaling_enabled=auto_scaling_enabled)
        self.check_no_transcoding = check_no_transcoding

        self.medialibrary._progressbar.connect(
            "notify::fraction", self._progress_bar_cb)

        self._create_assets(samples)
        self.mainloop.run()
        self.assertFalse(self.medialibrary._progressbar.props.visible)

    def check_add_proxy(self, asset, scaled=False, w=160, h=120,
                        check_progress=True):
        """Simulates the user requesting an asset to be proxied."""
        self.assertFalse(self.app.proxy_manager.is_proxy_asset(asset))

        # Check the initial state of the asset, nothing should be going on.
        self.assertNotIn("Proxy creation progress:",
                         self.medialibrary.store[0].infotext)
        self.assertIn(
            self.medialibrary.store[0].thumb_decorator.state,
            [AssetThumbnail.NO_PROXY, AssetThumbnail.UNSUPPORTED])

        # Check proxy creation.
        was_in_progress = False

        project = self.app.project_manager.current_project
        project.scaled_proxy_width = w
        project.scaled_proxy_height = h

        def check_set_state(self):
            old_set_state(self)
            if self.state == self.IN_PROGRESS:
                nonlocal was_in_progress
                was_in_progress = True

        old_set_state = AssetThumbnail._set_state
        AssetThumbnail._set_state = check_set_state
        try:
            project.use_proxies_for_assets([asset], scaled)

            self.assertIn("Proxy creation progress:",
                          self.medialibrary.store[0].infotext)

            self.mainloop.run(timeout_seconds=10)
        finally:
            AssetThumbnail._set_state = old_set_state

        if check_progress:
            self.assertTrue(was_in_progress)

        # Finally, check the final status of the asset after proxying.
        self.assertNotIn("Proxy creation progress:",
                         self.medialibrary.store[0].infotext)
        if scaled:
            self.assertEqual(
                self.medialibrary.store[0].thumb_decorator.state,
                AssetThumbnail.SCALED)
        else:
            self.assertEqual(
                self.medialibrary.store[0].thumb_decorator.state,
                AssetThumbnail.PROXIED)

        proxy = self.medialibrary.store[0].asset
        self.assertEqual(proxy.props.proxy_target.props.id, asset.props.id)
        # Check if the asset is video or not
        if w:
            stream = proxy.get_info().get_video_streams()[0]
            resolution = [stream.get_width(), stream.get_height()]
            if scaled:
                self.assertEqual(resolution, [w, h])

        return proxy

    def check_disable_proxy(self, proxy, asset, delete=False):
        self.assertFalse(self.app.proxy_manager.is_proxy_asset(asset))
        self.assertTrue(self.app.proxy_manager.is_proxy_asset(proxy))

        self.app.project_manager.current_project.disable_proxies_for_assets(
            [proxy], delete_proxy_file=delete)

        self.assertIsNone(asset.get_proxy())
        self.assertEqual(self.medialibrary.store[0].uri,
                         asset.props.id)

        self.assertEqual(os.path.exists(Gst.uri_get_location(proxy.props.id)),
                         not delete)


class TestMediaLibrary(BaseTestMediaLibrary):

    def stop_using_proxies(self, delete_proxies=False):
        sample_name = "30fps_numeroted_frames_red.mkv"
        self.check_import([sample_name])

        asset_uri = common.get_sample_uri(sample_name)
        proxy = self.medialibrary.store[0].asset

        self.assertEqual(proxy.props.proxy_target.props.id, asset_uri)

        project = self.app.project_manager.current_project
        self.assertIn(proxy, project.list_assets(GES.UriClip))
        project.disable_proxies_for_assets([proxy], delete_proxies)
        self.assertNotIn(proxy, project.list_assets(GES.UriClip))
        self.assertEqual(self.medialibrary.store.get_n_items(),
                         len(self.samples))

        self.assertEqual(self.medialibrary.store[0].uri,
                         asset_uri)

    def test_transcoding_and_reusing(self):
        sample_name = "30fps_numeroted_frames_red.mkv"
        with common.cloned_sample(sample_name):
            # Create proxies.
            self.check_import([sample_name])

            # Try to import again, checking that no transcoding is done.
            self.check_import([sample_name],
                              check_no_transcoding=True)

    def test_disable_proxies(self):
        sample_name = "30fps_numeroted_frames_red.mkv"
        with common.cloned_sample(sample_name):
            self.check_import([sample_name],
                              proxying_strategy=ProxyingStrategy.NOTHING,
                              check_no_transcoding=True)

    def test_save_project_with_removed_proxy(self):
        sample_name = "30fps_numeroted_frames_red.mkv"
        with common.cloned_sample(sample_name):
            self.check_import([sample_name])

            project = self.app.project_manager.current_project
            asset = GES.UriClipAsset.request_sync(common.get_sample_uri(sample_name))
            target = asset.get_proxy_target()
            self.assertEqual(set(project.list_assets(GES.Extractable)), set([target, asset]))

            # Remove the asset
            self.medialibrary.remove_assets_action.emit("activate", None)

            # Make sure that the project has not assets anymore
            self.assertEqual(project.list_assets(GES.Extractable), [])

            # Save the project and reload it, making sure there is no asset
            # in that new project
            project_uri = Gst.filename_to_uri(tempfile.NamedTemporaryFile().name)
            project.save(project.ges_timeline, project_uri, None, True)

            self._custom_set_up(project_uri)
            self.assertNotEqual(project, self.app.project_manager.current_project)
            project = self.app.project_manager.current_project
            self.assertEqual(project.list_assets(GES.Extractable), [])

    def check_selection_post_import(self, **kwargs):
        samples = ["30fps_numeroted_frames_red.mkv",
                   "30fps_numeroted_frames_blue.webm"]
        with common.cloned_sample(*samples):
            self.check_import(samples, **kwargs)
        self.assertEqual(len(list(self.medialibrary.get_selected_paths())),
                         len(self.samples))

    def test_newly_imported_asset_selected_optimize_all(self):
        self.check_selection_post_import(proxying_strategy=ProxyingStrategy.ALL)

    def test_newly_imported_asset_selected_optimize_automatic(self):
        self.check_selection_post_import(proxying_strategy=ProxyingStrategy.AUTOMATIC)

    def test_newly_imported_asset_selected_optimize_nothing(self):
        self.check_selection_post_import(proxying_strategy=ProxyingStrategy.NOTHING)

    def test_stop_using_proxies(self):
        sample_name = "30fps_numeroted_frames_red.mkv"
        with common.cloned_sample(sample_name):
            self.stop_using_proxies()

    def test_delete_proxy(self):
        sample_name = "30fps_numeroted_frames_red.mkv"
        with common.cloned_sample(sample_name):
            self.stop_using_proxies(delete_proxies=True)

            asset = self.medialibrary.store[0].asset
            proxy_uri = self.app.proxy_manager.get_proxy_uri(asset)

            # Requesting UriClip sync will return None if the asset is not in cache
            # this way we make sure that this asset used to exist
            proxy = GES.Asset.request(GES.UriClip, proxy_uri)
            self.assertIsNotNone(proxy)
            self.assertFalse(os.path.exists(Gst.uri_get_location(proxy_uri)))

            self.assertIsNone(asset.get_proxy())

            # And let's recreate the proxy file.
            self.app.project_manager.current_project.use_proxies_for_assets([asset])
            self.assertEqual(asset.creation_progress, 0)

            # Check that the info column notifies the user about progress
            self.assertTrue("Proxy creation progress:" in
                            self.medialibrary.store[0].infotext)

            # Run the mainloop and let _progress_bar_cb stop it when the proxy is
            # ready
            self.mainloop.run()

            self.assertEqual(asset.creation_progress, 100)
            self.assertEqual(asset.get_proxy(), proxy)

    def test_create_and_delete_scaled_proxy(self):
        sample_name = "30fps_numeroted_frames_red.mkv"
        with common.cloned_sample(sample_name):
            self.check_import([sample_name], proxying_strategy=ProxyingStrategy.NOTHING)
            asset = self.medialibrary.store[0].asset

            # Create scaled proxy
            proxy = self.check_add_proxy(asset, scaled=True)

            # Delete scaled proxy
            self.check_disable_proxy(proxy, asset, delete=True)

    def test_mixed_proxies(self):
        sample_name = "30fps_numeroted_frames_red.mkv"
        with common.cloned_sample(sample_name):
            self.check_import([sample_name], proxying_strategy=ProxyingStrategy.NOTHING)
            asset = self.medialibrary.store[0].asset

            # Create and disable scaled proxy
            proxy = self.check_add_proxy(asset, scaled=True)
            scaled_uri = self.app.proxy_manager.get_proxy_uri(asset, scaled=True)
            self.check_disable_proxy(proxy, asset)

            # Create and disable HQ proxy
            proxy = self.check_add_proxy(asset)
            hq_uri = self.app.proxy_manager.get_proxy_uri(asset)
            self.check_disable_proxy(proxy, asset)

            # Check both files exist
            self.assertTrue(os.path.exists(Gst.uri_get_location(hq_uri)))
            self.assertTrue(os.path.exists(Gst.uri_get_location(scaled_uri)))

            # Enable and delete scaled proxy
            proxy = self.check_add_proxy(asset, scaled=True,
                                         check_progress=False)
            self.check_disable_proxy(proxy, asset, delete=True)

            # Check that only HQ Proxy exists
            self.assertFalse(os.path.exists(Gst.uri_get_location(scaled_uri)))
            self.assertTrue(os.path.exists(Gst.uri_get_location(hq_uri)))

            # Enable and delete HQ proxy
            proxy = self.check_add_proxy(asset, check_progress=False)
            self.check_disable_proxy(proxy, asset, delete=True)

    def test_proxy_duration_mismatch(self):
        sample_name = "30fps_numeroted_frames_red.mkv"
        with common.cloned_sample(sample_name):
            self.check_import([sample_name], proxying_strategy=ProxyingStrategy.NOTHING)
            timeline = self.app.project_manager.current_project.ges_timeline

            asset = self.medialibrary.store[0].asset
            clip = timeline.append_layer().add_asset(asset, 0, 0, Gst.CLOCK_TIME_NONE, GES.TrackType.VIDEO)

            duration = 2.5 * Gst.SECOND
            fake_duration = 3 * Gst.SECOND

            asset.set_uint64(ASSET_DURATION_META, fake_duration)
            clip.props.max_duration = fake_duration
            clip.props.duration = fake_duration
            self.assertEqual(clip.props.duration, fake_duration)
            proxy = self.check_add_proxy(asset)

            self.assertEqual(asset_get_duration(asset), duration)
            self.assertEqual(asset_get_duration(proxy), duration)
            self.assertEqual(clip.props.duration, duration)
            self.assertEqual(clip.props.max_duration, duration)

    def test_timeline_proxy_switching(self):
        sample_name = "30fps_numeroted_frames_red.mkv"
        with common.cloned_sample(sample_name):
            self.check_import([sample_name], proxying_strategy=ProxyingStrategy.NOTHING)

            common.create_timeline_container(self.app)
            timeline = self.app.project_manager.current_project.ges_timeline

            asset = self.medialibrary.store[0].asset
            timeline.append_layer().add_asset(asset, 0, 0, Gst.CLOCK_TIME_NONE,
                                              GES.TrackType.VIDEO)

            def check_timeline_clip(expected_asset):
                for layer in timeline.layers:
                    for clip in layer.get_clips():
                        self.assertEqual(clip.get_asset(), expected_asset)

            # Check asset is in the timeline
            check_timeline_clip(asset)

            # Check asset is replaced by scaled proxy
            scaled_proxy = self.check_add_proxy(asset, scaled=True)
            check_timeline_clip(scaled_proxy)

            # Check proxy is replaced back by asset
            self.check_disable_proxy(scaled_proxy, asset, delete=True)
            check_timeline_clip(asset)

            # Check asset is replaced by HQ proxy
            hq_proxy = self.check_add_proxy(asset)
            check_timeline_clip(hq_proxy)

            # Check proxy was replaced back by asset
            self.check_disable_proxy(hq_proxy, asset, delete=True)
            check_timeline_clip(asset)

    def test_regenerate_scaled_proxy(self):
        sample_name = "30fps_numeroted_frames_red.mkv"
        with common.cloned_sample(sample_name):
            self.check_import([sample_name], proxying_strategy=ProxyingStrategy.NOTHING)
            asset = self.medialibrary.store[0].asset
            asset_uri = common.get_sample_uri(sample_name)

            # Create scaled proxy
            proxy = self.check_add_proxy(asset, scaled=True)
            proxy_uri = self.app.proxy_manager.get_proxy_uri(asset, scaled=True)

            # Change target resolution and trigger regeneration (1/4 Asset width)
            self.app.project_manager.current_project.scaled_proxy_width = 80
            self.app.project_manager.current_project.scaled_proxy_height = 60

            self.app.project_manager.current_project.regenerate_scaled_proxies()
            self.assertTrue("Proxy creation progress:" in
                            self.medialibrary.store[0].infotext)
            self.mainloop.run()

            proxy = self.medialibrary.store[0].asset
            self.assertNotEqual(proxy.props.id, proxy_uri)

            stream = proxy.get_info().get_video_streams()[0]
            resolution = [stream.get_width(), stream.get_height()]
            self.assertEqual(resolution, [80, 60])
            self.assertEqual(proxy.props.proxy_target.props.id, asset_uri)

            # Delete proxy
            self.check_disable_proxy(proxy, asset, delete=True)
            self.assertFalse(os.path.exists(Gst.uri_get_location(proxy_uri)))

    def test_scaled_proxy_for_unsupported_asset(self):
        sample_name = "1sec_simpsons_trailer.mp4"
        with common.cloned_sample(sample_name):
            self.check_import([sample_name], proxying_strategy=ProxyingStrategy.AUTOMATIC)
            asset = self.medialibrary.store[0].asset

            # Mark all formats as unsupported
            with mock.patch.object(self.app.proxy_manager,
                                   "is_asset_format_well_supported",
                                   return_value=False):
                # Create scaled proxy
                proxy = self.check_add_proxy(asset, scaled=True, w=80, h=34)
                proxy_uri = self.app.proxy_manager.get_proxy_uri(asset, scaled=True)
                self.mainloop.run(until_empty=True)

                # Check that HQ proxy was created
                hq_uri = self.app.proxy_manager.get_proxy_uri(asset)
                self.assertTrue(os.path.exists(Gst.uri_get_location(hq_uri)))

                # Delete scaled proxy
                self.check_disable_proxy(proxy, asset, delete=True)
                self.mainloop.run()

                # Check that we revert to HQ proxy
                proxy = self.medialibrary.store[0].asset
                proxy_uri = self.app.proxy_manager.get_proxy_uri(asset, scaled=False)
                self.assertEqual(proxy.props.id, proxy_uri)

                # Delete HQ Proxy
                self.check_disable_proxy(proxy, asset, delete=True)

    def test_load_audio_only(self):
        audio_sample = "mp3_sample.mp3"
        with common.cloned_sample(audio_sample):
            asset_uri = common.get_sample_uri(audio_sample)
            with common.created_project_file(asset_uri) as uri:
                self._custom_set_up(project_uri=uri)

    def test_scale_proxy_audio_post_import(self):
        sample = "mp3_sample.mp3"
        with common.cloned_sample(sample):
            self.check_import([sample], proxying_strategy=ProxyingStrategy.NOTHING)
            asset = self.medialibrary.store[0].asset

            project = self.app.project_manager.current_project
            project.use_proxies_for_assets([asset], scaled=True)

    def test_supported_out_of_container_audio(self):
        sample = "mp3_sample.mp3"
        with common.cloned_sample(sample):
            self.check_import([sample], check_no_transcoding=True,
                              proxying_strategy=ProxyingStrategy.AUTOMATIC)

    def test_import_supported_forced_scaled_audio(self):
        sample = "mp3_sample.mp3"
        with common.cloned_sample(sample):
            self.check_import([sample], auto_scaling_enabled=True)

    def test_missing_uri_displayed(self):
        asset_uri = common.get_sample_uri("image-which-does-not-exist.png")
        with common.created_project_file(asset_uri) as uri:
            self._custom_set_up(project_uri=uri)
        self.assertTrue(self.medialibrary._import_warning_infobar.props.visible)

    def test_right_click_on_media_library(self):
        self._custom_set_up()
        mlib = self.medialibrary

        # Release click
        event = create_event(Gdk.EventType.BUTTON_RELEASE, button=3)
        mlib._flowbox_button_release_event_cb(mlib.flowbox, event)


class TestTaggingAssets(BaseTestMediaLibrary):

    def import_assets_in_medialibrary(self):
        samples = ["30fps_numeroted_frames_red.mkv",
                   "30fps_numeroted_frames_blue.webm", "1sec_simpsons_trailer.mp4"]
        with common.cloned_sample(*samples):
            self.check_import(samples, proxying_strategy=ProxyingStrategy.NOTHING)

        self.assertTrue(self.medialibrary.tags_button.is_sensitive())

    def add_new_tag(self, tag_name):
        # Open the popover
        self.medialibrary.tags_button.props.active = True
        self.medialibrary.new_tag_entry.set_text(tag_name)
        self.medialibrary.new_tag_entry.emit("activate")
        self.medialibrary.tags_popover.hide()

    def get_tags_list(self):
        box = self.medialibrary.tags_popover.get_child()
        popover_widgets = box.get_children()
        return popover_widgets[1]

    def assert_tags_popover(self, tags_state):
        box = self.medialibrary.tags_popover.get_child()
        popover_widgets = box.get_children()
        tags_list = popover_widgets[1]

        for row_widget in tags_list:
            checkbox = row_widget.get_child()
            tag_name = checkbox.get_label()
            if tags_state[tag_name] == TagState.PRESENT:
                self.assertTrue(checkbox.props.active)
            elif tags_state[tag_name] == TagState.INCONSISTENT:
                self.assertTrue(checkbox.props.inconsistent)
            elif tags_state[tag_name] == TagState.ABSENT:
                self.assertFalse(checkbox.props.active)
            else:
                raise Exception(tag_name)

    def test_adding_tags(self):
        self.mainloop = common.create_main_loop()
        self.import_assets_in_medialibrary()
        self.medialibrary.flowbox.unselect_all()

        # Add a new tag "TAG" to asset1 via new tag entry field
        asset1 = self.medialibrary.flowbox.get_child_at_index(0)
        self.medialibrary.flowbox.select_child(asset1)
        tag = "TAG"
        self.add_new_tag(tag)
        self.medialibrary.tags_button.props.active = True
        self.assert_tags_popover({tag: TagState.PRESENT})
        self.medialibrary.tags_popover.hide()
        self.assertEqual(self.medialibrary.store[0].tags, {tag})
        self.assertEqual(self.medialibrary.witnessed_tags, {tag})
        self.medialibrary.flowbox.unselect_all()

        # Add an existing tag "TAG" to asset2 via toggling the unchecked mark
        asset2 = self.medialibrary.flowbox.get_child_at_index(1)
        self.medialibrary.flowbox.select_child(asset2)
        # Check if the tag is unchecked for now
        self.medialibrary.tags_button.props.active = True
        self.assert_tags_popover({tag: TagState.ABSENT})
        tags_list = self.get_tags_list()
        row_widget = tags_list.get_row_at_index(0)
        checkbox = row_widget.get_child()
        self.assertFalse(checkbox.props.active)
        checkbox.props.active = True
        self.medialibrary.tags_popover.hide()

        # Check if "TAG" is applied to asset2
        self.assertEqual(self.medialibrary.store[1].tags, {tag})
        # Reopen popover to check if "TAG" is present and correctly checked in asset2
        self.medialibrary.tags_button.props.active = True
        self.assert_tags_popover({tag: TagState.PRESENT})
        self.medialibrary.tags_popover.hide()

        # We have "TAG" for both asset1 and asset2, but asset3 has no tags
        # Add the existing inconsistent tag to every selected asset
        self.medialibrary.flowbox.select_all()
        self.medialibrary.tags_button.props.active = True
        self.assert_tags_popover({tag: TagState.INCONSISTENT})
        tags_list = self.get_tags_list()
        row_widget = tags_list.get_row_at_index(0)
        checkbox = row_widget.get_child()
        # Make the tag present in all the tags
        checkbox.props.active = True

        # Reopen the tags popover to check if "TAG" is present in all assets
        self.medialibrary.tags_popover.hide()
        # Check if "TAG" is present in all the 3 assets
        self.medialibrary.tags_button.props.active = True
        self.assert_tags_popover({tag: TagState.PRESENT})
        self.medialibrary.tags_popover.hide()

    def test_removing_tags(self):
        self.mainloop = common.create_main_loop()
        self.import_assets_in_medialibrary()
        tag = "TAG"
        self.add_new_tag(tag)
        self.assertEqual(self.medialibrary.witnessed_tags, {tag})

        # Make sure "TAG" is present in all the assets
        self.medialibrary.tags_button.props.active = True
        self.assert_tags_popover({tag: TagState.PRESENT})
        self.medialibrary.tags_popover.hide()
        self.medialibrary.flowbox.unselect_all()

        # Remove "TAG" from a single asset
        child = self.medialibrary.flowbox.get_child_at_index(0)
        self.medialibrary.flowbox.select_child(child)
        self.medialibrary.tags_button.props.active = True
        self.assertEqual(self.medialibrary.store[0].tags, {tag})
        tags_list = self.get_tags_list()
        row_widget = tags_list.get_row_at_index(0)
        checkbox = row_widget.get_child()
        self.assertTrue(checkbox.props.active)
        checkbox.props.active = False
        self.medialibrary.tags_popover.hide()

        # Confirm "TAG" is removed from asset1
        self.assertEqual(self.medialibrary.store[0].tags, set())
        # Reopen the tags popover to check if "TAG" is removed from asset1
        self.medialibrary.tags_button.props.active = True
        self.assert_tags_popover({tag: TagState.ABSENT})
        self.medialibrary.tags_popover.hide()

        # Remove inconsistent "TAG" from the selected assets
        self.medialibrary.flowbox.select_all()
        self.medialibrary.tags_button.props.active = True

        tags_list = self.get_tags_list()
        row_widget = tags_list.get_row_at_index(0)
        checkbox = row_widget.get_child()
        self.assertTrue(checkbox.props.inconsistent)
        # Toggle it to checked
        checkbox.props.active = True
        # Toggle it to unchecked
        checkbox.props.active = False
        self.medialibrary.tags_popover.hide()

        # Check if popover is empty
        self.medialibrary.tags_button.props.active = True
        tags_list = self.get_tags_list()
        row_widget = tags_list.get_row_at_index(0)
        self.assertIsNone(row_widget)
        self.medialibrary.tags_popover.hide()

        self.assertEqual(self.medialibrary.witnessed_tags, set())
        for item in self.medialibrary.store:
            self.assertEqual(item.tags, set())



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2015, Alex Băluț <alexandru.balut@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
from unittest import mock

from gi.repository import GES

from pitivi.timeline.layer import AUDIO_ICONS
from pitivi.timeline.layer import Layer
from pitivi.timeline.layer import VIDEO_ICONS
from tests import common


class TestLayerControl(common.TestCase):

    def test_name(self):
        timeline = mock.MagicMock()
        ges_layer = GES.Layer()
        layer = Layer(ges_layer, timeline)
        self.assertEqual(layer.get_name(), "Layer 0", "Default name generation failed")

        ges_layer.set_meta("audio::name", "a")
        self.assertEqual(layer.get_name(), "a", "Cannot use old audio name")

        ges_layer.set_meta("video::name", "v")
        self.assertEqual(layer.get_name(), "v", "Cannot use old video name")

        layer.set_name("vv")
        self.assertEqual(layer.get_name(), "vv")

    def test_name_meaningful(self):
        timeline = mock.MagicMock()
        ges_layer = GES.Layer()
        layer = Layer(ges_layer, timeline)
        layer.set_name("Layer 0x")
        self.assertEqual(layer.get_name(), "Layer 0x")

    def test_audio_toggle(self):
        """Checks that audio toggling is reflected in the UI."""
        timeline_container = common.create_timeline_container()
        timeline = timeline_container.timeline
        ges_layer = timeline.ges_timeline.append_layer()
        layer_controls = ges_layer.control_ui
        audio_button = layer_controls.audio_button

        for audio_track in layer_controls.timeline_audio_tracks:
            self.assertTrue(ges_layer.get_active_for_track(audio_track))
            self.assertEqual(audio_button.get_image().props.icon_name, AUDIO_ICONS[True])

            ges_layer.set_active_for_tracks(False, [audio_track])
            common.create_main_loop().run(until_empty=True)
            self.assertFalse(ges_layer.get_active_for_track(audio_track))
            self.assertEqual(audio_button.get_image().props.icon_name, AUDIO_ICONS[False])

            ges_layer.set_active_for_tracks(True, [audio_track])
            common.create_main_loop().run(until_empty=True)
            self.assertTrue(ges_layer.get_active_for_track(audio_track))
            self.assertEqual(audio_button.get_image().props.icon_name, AUDIO_ICONS[True])

    def test_audio_button(self):
        """Checks that the audio button toggles the audio track."""
        timeline_container = common.create_timeline_container()
        timeline = timeline_container.timeline
        ges_layer = timeline.ges_timeline.append_layer()
        layer_controls = ges_layer.control_ui
        audio_button = layer_controls.audio_button

        for audio_track in layer_controls.timeline_audio_tracks:
            self.assertTrue(ges_layer.get_active_for_track(audio_track))
        common.create_main_loop().run(until_empty=True)
        self.assertEqual(audio_button.get_image().props.icon_name, AUDIO_ICONS[True])

        audio_button.clicked()
        for audio_track in layer_controls.timeline_audio_tracks:
            self.assertFalse(ges_layer.get_active_for_track(audio_track))
        common.create_main_loop().run(until_empty=True)
        self.assertEqual(audio_button.get_image().props.icon_name, AUDIO_ICONS[False])

        audio_button.clicked()
        for audio_track in layer_controls.timeline_audio_tracks:
            self.assertTrue(ges_layer.get_active_for_track(audio_track))
        common.create_main_loop().run(until_empty=True)
        self.assertEqual(audio_button.get_image().props.icon_name, AUDIO_ICONS[True])

    def test_video_toggle(self):
        """Checks that video toggling is reflected in the UI."""
        timeline_container = common.create_timeline_container()
        timeline = timeline_container.timeline
        ges_layer = timeline.ges_timeline.append_layer()
        layer_controls = ges_layer.control_ui
        video_button = layer_controls.video_button

        for video_track in layer_controls.timeline_video_tracks:
            self.assertTrue(ges_layer.get_active_for_track(video_track))
            self.assertEqual(video_button.get_image().props.icon_name, VIDEO_ICONS[True])

            ges_layer.set_active_for_tracks(False, [video_track])
            common.create_main_loop().run(until_empty=True)
            self.assertFalse(ges_layer.get_active_for_track(video_track))
            self.assertEqual(video_button.get_image().props.icon_name, VIDEO_ICONS[False])

            ges_layer.set_active_for_tracks(True, [video_track])
            common.create_main_loop().run(until_empty=True)
            self.assertTrue(ges_layer.get_active_for_track(video_track))
            self.assertEqual(video_button.get_image().props.icon_name, VIDEO_ICONS[True])

    def test_video_button(self):
        """Checks that the video button toggles the video track."""
        timeline_container = common.create_timeline_container()
        timeline = timeline_container.timeline
        ges_layer = timeline.ges_timeline.append_layer()
        layer_controls = ges_layer.control_ui
        video_button = layer_controls.video_button

        for video_track in layer_controls.timeline_video_tracks:
            self.assertTrue(ges_layer.get_active_for_track(video_track))
        common.create_main_loop().run(until_empty=True)
        self.assertEqual(video_button.get_image().props.icon_name, VIDEO_ICONS[True])

        video_button.clicked()
        for video_track in layer_controls.timeline_video_tracks:
            self.assertFalse(ges_layer.get_active_for_track(video_track))
        common.create_main_loop().run(until_empty=True)
        self.assertEqual(video_button.get_image().props.icon_name, VIDEO_ICONS[False])

        video_button.clicked()
        for video_track in layer_controls.timeline_video_tracks:
            self.assertTrue(ges_layer.get_active_for_track(video_track))
        common.create_main_loop().run(until_empty=True)
        self.assertEqual(video_button.get_image().props.icon_name, VIDEO_ICONS[True])


class TestLayer(common.TestCase):

    def test_check_media_types(self):
        """Checks media types when there is no control UI."""
        ges_layer = GES.Layer()
        png = common.get_sample_uri("flat_colour1_640x480.png")
        video_clip = GES.UriClipAsset.request_sync(png).extract()
        self.assertTrue(ges_layer.add_clip(video_clip))
        self.assertEqual(len(ges_layer.get_clips()), 1)
        timeline_container = common.create_timeline_container()
        timeline = timeline_container.timeline
        # This will add widgets for the clips in ges_layer and
        # the layer will use check_media_types which updates the
        # height of layer.control_ui, which now it should not be set.
        self.assertFalse(hasattr(ges_layer, "control_ui"))
        unused_layer = Layer(ges_layer, timeline)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2011, Alex Băluț <alexandru.balut@gmail.com>
# Copyright (c) 2011, Jean-François Fortin Tam <nekohayo@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the pitivi.preset module."""
# pylint: disable=protected-access
import os.path
import shutil
import tempfile

from pitivi.preset import AudioPresetManager
from pitivi.preset import PresetManager
from pitivi.utils.system import System
from tests import common


def clear_preset_manager_paths(preset_manager):
    try:
        shutil.rmtree(preset_manager.user_path)
    except FileNotFoundError:
        pass


def count_json_files_in(dir_path):
    return len([filename
                for filename in os.listdir(dir_path)
                if filename.endswith(".json")])


def count_default_presets(preset_manager):
    return count_json_files_in(preset_manager.default_path)


def count_user_presets(preset_manager):
    return count_json_files_in(preset_manager.user_path)


class TestPresetBasics(common.TestCase):

    def setUp(self):
        self.manager = PresetManager(None, tempfile.mkdtemp(), System())
        self.manager._serialize_preset = lambda preset: dict(preset.items())

    def tearDown(self):
        clear_preset_manager_paths(self.manager)

    def test_add_preset(self):
        self.manager.create_preset('preseT onE', {'name1': '1A'})
        self.manager.create_preset('Preset One', {'name1': '2A'})
        self.assertEqual(2, len(self.manager.presets))

    def test_add_preset_with_non_ascii_name(self):
        unicode_name = "ソリッド・スネーク"
        self.manager.create_preset(unicode_name, {})
        self.assertTrue(unicode_name in self.manager.get_preset_names())

    def test_rename_preset(self):
        self.manager.create_preset('preseT onE', {'name1': '1A'})
        self.manager.create_preset('Preset Two', {'name1': '2A'})
        self.assertEqual(2, len(self.manager.presets))

        self.manager.restore_preset('preseT onE')
        self.manager.save_current_preset('Preset One')
        self.assertEqual(2, len(self.manager.presets))
        self.manager.save_current_preset('Preset TWO')
        self.assertEqual(2, len(self.manager.presets))
        self.manager.save_current_preset('Preset two')
        self.assertEqual(2, len(self.manager.presets))

        self.manager.save_current_preset('Preset Two')
        self.assertEqual(1, len(self.manager.presets))
        self.manager.save_current_preset('Preset Two')
        self.assertEqual(1, len(self.manager.presets))
        self.manager.save_current_preset('preseT onE')
        self.assertEqual(1, len(self.manager.presets))

    def test_load_handles_missing_directory(self):
        self.manager.default_path = '/pitivi/non/existing/directory/1'
        self.manager.user_path = '/pitivi/non/existing/directory/2'
        self.manager.load_all()

    def test_get_unique_preset_name(self):
        name = self.manager.get_new_preset_name()
        self.assertEqual('New preset', name)

        self.manager.create_preset(name, {})
        new_preset1 = self.manager.get_new_preset_name()
        self.assertEqual('New preset 1', new_preset1)

        # Intentionally add 'New preset 2' before 'New preset 1'.
        self.manager.create_preset('New preset 2', {})
        self.manager.create_preset('New preset 1', {})
        new_preset3 = self.manager.get_new_preset_name()
        self.assertEqual('New preset 3', new_preset3)


class TestAudioPresetsIO(common.TestCase):

    def setUp(self):
        self.manager = AudioPresetManager(System())
        self.manager.user_path = tempfile.mkdtemp()

    def tearDown(self):
        clear_preset_manager_paths(self.manager)

    def create_other_manager(self):
        other_manager = AudioPresetManager(System())
        other_manager.user_path = self.manager.user_path
        return other_manager

    def test_save_and_load(self):
        self.manager.create_preset("Vegeta",
                                   {"channels": 6000,
                                    "sample-rate": 44100})
        self.manager.save_all()
        self.assertEqual(1, count_user_presets(self.manager))

        self.manager.create_preset("Nappa",
                                   {"channels": 4000,
                                    "sample-rate": 44100})
        self.manager.save_all()
        self.assertEqual(2, count_user_presets(self.manager))

        other_manager = self.create_other_manager()
        other_manager.load_all()
        total_presets = count_default_presets(
            self.manager) + count_user_presets(self.manager)
        self.assertEqual(total_presets, len(other_manager.presets))

    def test_non_ascii_filenames_save_and_load(self):
        non_ascii_preset_name = "Solid Snake (ソリッド・スネーク) \\#!\"'$%?&*"
        self.manager.create_preset(non_ascii_preset_name,
                                   {"channels": 2,
                                    "sample-rate": 44100})
        snake = self.manager.presets[non_ascii_preset_name]
        self.assertEqual(2, len(snake))
        self.manager.save_all()

        other_manager = self.create_other_manager()
        other_manager.load_all()
        self.assertEqual(1 + count_default_presets(
            other_manager), len(other_manager.presets))
        snaaaake = other_manager.presets[non_ascii_preset_name]
        self.assertEqual(snake, snaaaake)

    def test_invalid_filenames_save_and_load(self):
        # This would be an invalid file name as is.
        preset_name = " / % "
        self.manager.create_preset(preset_name,
                                   {"channels": 2,
                                    "sample-rate": 44100})
        values = self.manager.presets[preset_name]
        self.assertEqual(2, len(values))
        self.manager.save_all()

        other_manager = self.create_other_manager()
        other_manager.load_all()
        self.assertEqual(1 + count_default_presets(
            other_manager), len(other_manager.presets))
        other_values = other_manager.presets[preset_name]
        self.assertEqual(values, other_values)

    def test_removing_system_presets(self):
        self.manager.load_all()
        system_presets = list(self.manager.presets.keys())
        for preset_name in system_presets:
            self.manager.restore_preset(preset_name)
            self.manager.remove_current_preset()

        # Check that the files have not been deleted or changed.
        other_manager = AudioPresetManager(System())
        other_manager.user_path = "/pitivi/non/existing/directory"
        other_manager.load_all()
        for preset_name in system_presets:
            self.assertTrue(other_manager.has_preset(preset_name))

        # Check that overwrite files have been created and
        # they mark the system presets as deleted.
        other_manager = self.create_other_manager()
        other_manager.load_all()
        for preset_name in system_presets:
            self.assertFalse(other_manager.has_preset(preset_name))

    def test_renaming_system_presets(self):
        self.manager.load_all()
        system_presets = list(self.manager.presets.keys())
        new_name_template = "%s new"
        for preset_name in system_presets:
            self.manager.restore_preset(preset_name)
            new_name = new_name_template % preset_name
            self.manager.save_current_preset(new_name)

        # Check that the files have not been deleted or changed.
        other_manager = AudioPresetManager(System())
        other_manager.user_path = "/pitivi/non/existing/directory"
        other_manager.load_all()
        for preset_name in system_presets:
            self.assertTrue(other_manager.has_preset(preset_name), preset_name)

        other_manager = self.create_other_manager()
        other_manager.load_all()
        for preset_name in system_presets:
            self.assertFalse(other_manager.has_preset(preset_name), preset_name)
            new_name = new_name_template % preset_name
            self.assertTrue(other_manager.has_preset(new_name), new_name)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2010 Mathieu Duponchelle <seeed@laposte.net>
# Copyright (c) 2018 Harish Fulara <harishfulara1996@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Pitivi's Welcome/Greeter perspective."""
import os
from gettext import gettext as _

from gi.repository import Gdk
from gi.repository import GES
from gi.repository import Gio
from gi.repository import GLib
from gi.repository import Gtk

from pitivi.check import MISSING_SOFT_DEPS
from pitivi.configure import get_ui_dir
from pitivi.dialogs.browseprojects import BrowseProjectsDialog
from pitivi.perspective import Perspective
from pitivi.project import Project
from pitivi.utils.ui import beautify_last_updated_timestamp
from pitivi.utils.ui import beautify_project_path
from pitivi.utils.ui import BinWithNaturalWidth
from pitivi.utils.ui import fix_infobar
from pitivi.utils.ui import GREETER_PERSPECTIVE_CSS
from pitivi.utils.ui import PADDING
from pitivi.utils.ui import SPACING
from pitivi.utils.ui import URI_TARGET_ENTRY

MAX_RECENT_PROJECTS = 10


class ProjectInfoRow(Gtk.ListBoxRow):
    """Displays a project's info.

    Attributes:
        recent_project_item (Gtk.RecentInfo): Recent project's meta-data.
    """

    def __init__(self, recent_project_item):
        Gtk.ListBoxRow.__init__(self)
        self.uri = recent_project_item.get_uri()
        self.name = os.path.splitext(recent_project_item.get_display_name())[0]

        builder = Gtk.Builder()
        builder.add_from_file(os.path.join(get_ui_dir(), "project_info.ui"))
        self.add(builder.get_object("project_info_tophbox"))

        self.select_button = builder.get_object("project_select_button")
        # Hide the select button as we only want to
        # show it during projects removal screen.
        self.select_button.hide()

        self.__thumb = builder.get_object("project_thumbnail")
        # Defer loading of thumbnail.
        GLib.idle_add(self.__load_thumb_cb)

        builder.get_object("project_name_label").set_text(self.name)
        builder.get_object("project_uri_label").set_text(
            beautify_project_path(recent_project_item.get_uri_display()))
        builder.get_object("project_last_updated_label").set_text(
            beautify_last_updated_timestamp(recent_project_item.get_modified()))

    def __load_thumb_cb(self):
        self.__thumb.set_from_pixbuf(Project.get_thumb(self.uri))
        return False


class GreeterPerspective(Perspective):
    """Pitivi's Welcome/Greeter perspective.

    Allows the user to create a new project or open an existing one.

    Attributes:
        app (Pitivi): The app.
    """

    def __init__(self, app):
        Perspective.__init__(self)

        self.app = app
        self.new_project_action = None
        self.open_project_action = None

        self.__topvbox = None
        self.__welcome_vbox = None
        self.__recent_projects_vbox = None
        self.__search_entry = None
        self.__recent_projects_labelbox = None
        self.__recent_projects_listbox = None
        self.__project_filter = self.__create_project_filter()
        self.__infobar = None
        self.__selection_button = None
        self.__actionbar = None
        self.__remove_projects_button = None
        self.__cancel_button = None
        self.__new_project_button = None
        self.__open_project_button = None
        self.__warnings_button = None

        # Projects selected for removal.
        self.__selected_projects = []

        if app.get_latest():
            self.__show_newer_available_version()
        else:
            app.connect("version-info-received", self.__app_version_info_received_cb)

    def setup_ui(self):
        """Sets up the UI."""
        builder = Gtk.Builder()
        builder.add_from_file(os.path.join(get_ui_dir(), "greeter.ui"))

        logo = builder.get_object("logo")
        icon_theme = Gtk.IconTheme.get_default()
        pixbuf = icon_theme.load_icon("org.pitivi.Pitivi", 256, Gtk.IconLookupFlags.FORCE_SIZE)
        logo.set_from_pixbuf(pixbuf)

        self.toplevel_widget = builder.get_object("toplevel_vbox")
        self.toplevel_widget.drag_dest_set(
            Gtk.DestDefaults.ALL, [URI_TARGET_ENTRY], Gdk.DragAction.COPY)
        self.toplevel_widget.connect("drag-data-received", self.__drag_data_received_cb)

        self.__topvbox = builder.get_object("topvbox")
        self.__welcome_vbox = builder.get_object("welcome_vbox")
        self.__recent_projects_vbox = builder.get_object("recent_projects_vbox")

        self.__recent_projects_labelbox = builder.get_object("recent_projects_labelbox")

        self.__search_entry = builder.get_object("search_entry")
        self.__search_entry.connect("search-changed", self.__search_changed_cb)

        self.__recent_projects_listbox = builder.get_object("recent_projects_listbox")
        self.__recent_projects_listbox.set_selection_mode(Gtk.SelectionMode.NONE)
        self.__recent_projects_listbox.connect(
            "row_activated", self.__projects_row_activated_cb)
        self.__recent_projects_listbox.connect(
            "button-press-event", self.__projects_button_press_cb)

        self.__infobar = builder.get_object("infobar")
        fix_infobar(self.__infobar)
        self.__infobar.hide()
        self.__infobar.connect("response", self.__infobar_response_cb)

        self.__actionbar = builder.get_object("actionbar")
        self.__remove_projects_button = builder.get_object("remove_projects_button")
        self.__remove_projects_button.get_style_context().add_class("destructive-action")
        self.__remove_projects_button.connect("clicked", self.__remove_projects_clicked_cb)

        self.__setup_css()
        self.headerbar = self.__create_headerbar()
        self._create_actions()

    def refresh(self):
        """Refreshes the perspective."""
        # Hide actionbar because we only want to show it during projects removal screen.
        self.__actionbar.hide()
        self.__remove_projects_button.set_sensitive(False)
        self.__selected_projects = []

        # Clear the currently displayed list of recent projects.
        for child in self.__recent_projects_listbox.get_children():
            self.__recent_projects_listbox.remove(child)

        recent_items = [item for item in self.app.recent_manager.get_items()
                        if item.get_display_name().endswith(self.__project_filter) and item.exists()]

        # If there are recent projects, display them, else display welcome screen.
        if recent_items:
            for item in recent_items[:MAX_RECENT_PROJECTS]:
                recent_project_info = ProjectInfoRow(item)
                recent_project_info.select_button.connect(
                    "toggled", self.__project_selected_cb, recent_project_info)
                self.__recent_projects_listbox.add(recent_project_info)
                recent_project_info.show()

            child = self.__recent_projects_vbox
            self.__update_headerbar(projects=True)
            self.__recent_projects_listbox.show()
        else:
            child = self.__welcome_vbox
            self.__update_headerbar(welcome=True)

        children = self.__topvbox.get_children()
        if children:
            current_child = children[0]
            if current_child == child:
                child = None
            else:
                self.__topvbox.remove(current_child)

        if child:
            self.__topvbox.pack_start(child, False, False, 0)

        if recent_items:
            self.__search_entry.show()
            # We are assuming that the users name their projects meaningfully
            # and are sure of what project they want to search for. Once they
            # find the project and open it they don't want to come back to the
            # previous search results. So, we clear out the search entry before
            # the greeter is shown again.
            self.__search_entry.set_text("")
            self.__search_entry.grab_focus()

    def __setup_css(self):
        css_provider = Gtk.CssProvider()
        css_provider.load_from_data(GREETER_PERSPECTIVE_CSS.encode('UTF-8'))
        screen = Gdk.Screen.get_default()
        style_context = self.app.gui.get_style_context()
        style_context.add_provider_for_screen(screen, css_provider,
                                              Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)

    def __create_headerbar(self):
        headerbar = Gtk.HeaderBar()

        self.__new_project_button = Gtk.Button.new_with_label(_("New"))
        self.__new_project_button.set_tooltip_text(_("Create a new project"))
        self.__new_project_button.set_action_name("greeter.new-project")

        self.__open_project_button = Gtk.Button.new_with_label(_("Open…"))
        self.__open_project_button.set_tooltip_text(_("Open an existing project"))
        self.__open_project_button.set_action_name("greeter.open-project")

        self.__selection_button = Gtk.Button.new_from_icon_name("object-select-symbolic",
                                                                Gtk.IconSize.BUTTON)
        self.__selection_button.set_tooltip_text(_("Select projects for removal"))
        self.__selection_button.connect("clicked", self.__selection_clicked_cb)

        self.__cancel_button = Gtk.Button.new_with_label(_("Cancel"))
        self.__cancel_button.connect("clicked", self.__cancel_clicked_cb)

        self.__warnings_button = Gtk.MenuButton.new()
        self.__warnings_button.props.image = Gtk.Image.new_from_icon_name("warning-symbolic", Gtk.IconSize.BUTTON)
        self.__warnings_button.set_popover(self.__create_warnings_popover())

        self.menu_button = self.__create_menu()

        headerbar.pack_start(self.__new_project_button)
        headerbar.pack_start(self.__open_project_button)
        headerbar.pack_end(self.menu_button)
        headerbar.pack_end(self.__selection_button)
        headerbar.pack_end(self.__cancel_button)
        headerbar.pack_end(self.__warnings_button)
        headerbar.show()

        return headerbar

    def __update_headerbar(self, welcome=False, projects=False, selection=False):
        """Updates the headerbar depending on the greeter state."""
        self.__new_project_button.set_visible(welcome or projects)
        self.__open_project_button.set_visible(welcome or projects)
        self.__cancel_button.set_visible(selection)
        self.__selection_button.set_visible(projects)
        self.menu_button.set_visible(welcome or projects)
        self.headerbar.set_show_close_button(welcome or projects)
        self.__warnings_button.set_visible((welcome or projects) and MISSING_SOFT_DEPS)

        if selection:
            self.headerbar.get_style_context().add_class("selection-mode")
            self.headerbar.set_title(_("Click an item to select"))
        else:
            self.headerbar.get_style_context().remove_class("selection-mode")
            if projects:
                self.headerbar.set_title(_("Select a Project"))
            else:
                self.headerbar.set_title(_("Pitivi"))

    def _create_actions(self):
        group = Gio.SimpleActionGroup()
        self.toplevel_widget.insert_action_group("greeter", group)
        self.headerbar.insert_action_group("greeter", group)

        self.new_project_action = Gio.SimpleAction.new("new-project", None)
        self.new_project_action.connect("activate", self.__new_project_cb)
        group.add_action(self.new_project_action)
        self.app.shortcuts.add("greeter.new-project", ["<Primary>n"],
                               self.new_project_action,
                               _("Create a new project"), group="win")

        self.open_project_action = Gio.SimpleAction.new("open-project", None)
        self.open_project_action.connect("activate", self.__open_project_cb)
        group.add_action(self.open_project_action)
        self.app.shortcuts.add("greeter.open-project", ["<Primary>o"],
                               self.open_project_action,
                               _("Open a project"), group="win")

    @staticmethod
    def __create_project_filter():
        filter_ = []
        for asset in GES.list_assets(GES.Formatter):
            filter_.append(asset.get_meta(GES.META_FORMATTER_EXTENSION))
        return tuple(filter_)

    @staticmethod
    def __create_menu():
        builder = Gtk.Builder()
        builder.add_from_file(os.path.join(get_ui_dir(), "mainmenubutton.ui"))
        menu_button = builder.get_object("menubutton")
        # Menu options we want to display.
        visible_options = ["menu_shortcuts", "menu_help", "menu_about"]
        for widget in builder.get_object("menu_box").get_children():
            if Gtk.Buildable.get_name(widget) not in visible_options:
                widget.hide()
            else:
                visible_options.remove(Gtk.Buildable.get_name(widget))
        assert not visible_options
        return menu_button

    def __drag_data_received_cb(self, unused_widget, unused_context, unused_x,
                                unused_y, data, unused_info, unused_time):
        """Opens the project file dragged from Nautilus."""
        uris = data.get_uris()
        if not uris:
            return

        uri = uris[0]
        extension = os.path.splitext(uri)[1][1:]
        if extension in self.__project_filter:
            self.app.project_manager.load_project(uri)

    def __new_project_cb(self, unused_action, unused_param):
        self.app.project_manager.new_blank_project()

    def __open_project_cb(self, unused_action, unused_param):
        dialog = BrowseProjectsDialog(self.app)
        response = dialog.run()
        uri = dialog.get_uri()
        dialog.destroy()
        if response == Gtk.ResponseType.OK:
            self.app.project_manager.load_project(uri)

    def __app_version_info_received_cb(self, app, unused_version_information):
        """Handles new version info."""
        if app.is_latest():
            # current version, don't show message
            return
        self.__show_newer_available_version()

    def __show_newer_available_version(self):
        latest_version = self.app.get_latest()

        if self.app.settings.lastCurrentVersion != latest_version:
            # new latest version, reset counter
            self.app.settings.lastCurrentVersion = latest_version
            self.app.settings.displayCounter = 0

        if self.app.settings.displayCounter >= 5:
            # current version info already showed 5 times, don't show again
            return

        # increment counter, create infobar and show info
        self.app.settings.displayCounter += 1
        text = _("Pitivi %s is available.") % latest_version
        label = Gtk.Label(label=text)
        self.__infobar.get_content_area().add(label)
        self.__infobar.set_message_type(Gtk.MessageType.INFO)
        self.__infobar.show_all()

    def __infobar_response_cb(self, unused_infobar, response_id):
        if response_id == Gtk.ResponseType.CLOSE:
            self.__infobar.hide()

    def __projects_row_activated_cb(self, unused_listbox, row):
        if row.select_button.get_visible():
            row.select_button.set_active(not row.select_button.get_active())
        else:
            self.app.project_manager.load_project(row.uri)

    def __projects_button_press_cb(self, listbox, event):
        if event.button == 3:
            self.__start_selection_mode()
            row = listbox.get_row_at_y(event.y)
            if row:
                row.select_button.set_active(True)

    def __search_changed_cb(self, search_entry):
        search_hit = False
        search_text = search_entry.get_text().lower()
        style_context = search_entry.get_style_context()

        for recent_project_item in self.__recent_projects_listbox.get_children():
            if search_text in recent_project_item.name.lower():
                recent_project_item.show()
                search_hit = True
            else:
                recent_project_item.hide()

        if not search_hit:
            style_context.add_class("error")
        else:
            style_context.remove_class("error")

        self.__recent_projects_labelbox.set_visible(search_hit)
        self.__recent_projects_listbox.set_visible(search_hit)

    def __selection_clicked_cb(self, unused_button):
        self.__start_selection_mode()

    def __start_selection_mode(self):
        if self.__actionbar.get_visible():
            return

        self.__update_headerbar(selection=True)
        self.__search_entry.hide()
        self.__actionbar.show()
        for child in self.__recent_projects_listbox.get_children():
            child.select_button.show()

    def __cancel_clicked_cb(self, unused_button):
        self.refresh()

    def __project_selected_cb(self, check_button, project):
        if check_button.get_active():
            self.__selected_projects.append(project)
        else:
            self.__selected_projects.remove(project)

        self.__remove_projects_button.set_sensitive(bool(self.__selected_projects))

    def __remove_projects_clicked_cb(self, unused_button):
        for project in self.__selected_projects:
            self.app.recent_manager.remove_item(project.uri)
        self.refresh()

    def __create_warnings_popover(self):
        """Creates a popover listing missing soft dependencies."""
        popover = Gtk.Popover()
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, margin=PADDING * 3)

        label = Gtk.Label(_("To enable additional features, please install the following packages and restart Pitivi:"))
        label.props.halign = Gtk.Align.START
        label.props.wrap = True
        label.props.xalign = 0
        box.pack_start(label, False, False, 0)

        grid = Gtk.Grid()
        grid.props.row_spacing = SPACING
        grid.props.column_spacing = PADDING
        grid.props.margin_left = SPACING
        grid.props.margin_top = SPACING * 2

        for row_index, dep in enumerate(MISSING_SOFT_DEPS.values()):
            name_label = Gtk.Label(dep.modulename)
            name_label.props.selectable = True
            name_label.props.can_focus = False
            name_label.props.xalign = 0
            name_label.props.valign = Gtk.Align.START
            grid.attach(name_label, 0, row_index, 1, 1)

            mdash_label = Gtk.Label("―")
            mdash_label.props.xalign = 0
            mdash_label.props.valign = Gtk.Align.START
            grid.attach(mdash_label, 1, row_index, 1, 1)

            description_label = Gtk.Label(dep.additional_message)
            description_label.props.wrap = True
            description_label.props.xalign = 0
            description_label.props.yalign = Gtk.Align.START
            grid.attach(description_label, 2, row_index, 1, 1)

        box.pack_start(grid, False, False, 0)

        wrapper_bin = BinWithNaturalWidth(box, width=500)
        wrapper_bin.show_all()
        popover.add(wrapper_bin)
        return popover



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2014, Alex Băluț <alexandru.balut@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the application module."""
# pylint: disable=protected-access,no-self-use
import tempfile
from unittest import mock

from gi.repository import Gst

from pitivi import application
from pitivi import configure
from tests import common


class TestPitivi(common.TestCase):

    def call_version_info_received(self, version_info):
        app = application.Pitivi()
        giofile = mock.Mock()
        giofile.load_contents_finish.return_value = (True, version_info)
        app._version_info_received_cb(giofile, result=None, user_data=None)
        return app

    def test_version_info(self):
        app = application.Pitivi()
        self.assertTrue(app.is_latest())

        app = self.call_version_info_received("invalid")
        self.assertTrue(app.is_latest())

        app = self.call_version_info_received(
            "%s=CURRENT" % configure.VERSION)
        self.assertTrue(app.is_latest())
        self.assertEqual(configure.VERSION, app.get_latest())

        app = self.call_version_info_received(
            "%s=current\n0=supported" % configure.VERSION)
        self.assertTrue(app.is_latest())
        self.assertEqual(configure.VERSION, app.get_latest())

        app = self.call_version_info_received("0.91=current")
        self.assertTrue(app.is_latest())
        self.assertEqual("0.91", app.get_latest())

        app = self.call_version_info_received("9999.00000000=current")
        self.assertFalse(app.is_latest())
        self.assertEqual("9999.00000000", app.get_latest())

    def test_inhibition(self):
        app = application.Pitivi()

        # Check simple_inhibit.
        with mock.patch.object(app, "inhibit") as inhibit_mock:
            inhibit_mock.return_value = 1
            app.simple_inhibit("reason1", "flags1")
            inhibit_mock.return_value = 2
            app.simple_inhibit("reason2", "flags2")
            self.assertEqual(inhibit_mock.call_count, 2)

            inhibit_mock.reset_mock()
            app.simple_inhibit("reason1", "flags1.1")
            self.assertFalse(inhibit_mock.called)

        # Check simple_uninhibit.
        with mock.patch.object(app, "uninhibit") as uninhibit_mock:
            uninhibit_mock.reset_mock()
            app.simple_uninhibit("reason1")
            uninhibit_mock.assert_called_once_with(1)

            uninhibit_mock.reset_mock()
            app.simple_uninhibit("reason1")
            self.assertFalse(uninhibit_mock.called)

            uninhibit_mock.reset_mock()
            app.simple_uninhibit("reason2")
            uninhibit_mock.assert_called_once_with(2)

            uninhibit_mock.reset_mock()
            app.simple_uninhibit("reason2")
            self.assertFalse(uninhibit_mock.called)

            app.simple_uninhibit("reason3")
            self.assertFalse(uninhibit_mock.called)

        # Check again simple_inhibit.
        with mock.patch.object(app, "inhibit") as inhibit_mock:
            app.simple_inhibit("reason1", "flags1")
            self.assertTrue(inhibit_mock.called)

    def test_loading_missing_file(self):
        app = common.create_pitivi()
        project_uri = Gst.filename_to_uri(tempfile.NamedTemporaryFile().name)
        app.project_manager.load_project(project_uri)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2009, Brandon Lewis <brandon_lewis@berkeley.edu>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
from unittest import mock

from gi.repository import Gtk

from pitivi.dialogs.prefs import PreferencesDialog
from tests import common


class PreferencesDialogTest(common.TestCase):

    def test_dialog_creation(self):
        """Exercises the dialog creation."""
        app = common.create_pitivi()
        with mock.patch.object(Gtk.Dialog, "set_transient_for"):
            PreferencesDialog(app)

        app.project_manager.new_blank_project()
        with mock.patch.object(Gtk.Dialog, "set_transient_for"):
            PreferencesDialog(app)

    def test_numeric(self):
        PreferencesDialog.add_numeric_preference('numericPreference1',
                                                 label="Open Range",
                                                 section="timeline",
                                                 description="This option has no upper bound",
                                                 lower=-10)
        self.assertTrue(
            'numericPreference1' in PreferencesDialog.prefs["timeline"])

        PreferencesDialog.add_numeric_preference('numericPreference2',
                                                 label="Closed Range",
                                                 section="timeline",
                                                 description="This option has both upper and lower bounds",
                                                 lower=-10,
                                                 upper=10000)

    def test_text(self):
        PreferencesDialog.add_text_preference('textPreference1',
                                              label="Unfiltered",
                                              section="timeline",
                                              description="Anything can go in this box")

        PreferencesDialog.add_text_preference('textPreference2',
                                              label="Numbers only",
                                              section="timeline",
                                              description="This input validates its input with a regex",
                                              matches=r"^-?\d+(\.\d+)?$")

    def test_other(self):
        PreferencesDialog.add_path_preference('aPathPreference',
                                              label="Test Path",
                                              section="timeline",
                                              description="Test the path widget")

        PreferencesDialog.add_choice_preference('aChoicePreference',
                                                label="Swallow Velocity",
                                                section="timeline",
                                                description="What is the airspeed velocity of a coconut-laden swallow?",
                                                choices=(
                                                    ("42 Knots", 32),
                                                    ("9 furlongs per fortnight", 42),
                                                    ("I don't know that!", None)))

        PreferencesDialog.add_choice_preference('aLongChoicePreference',
                                                label="Favorite Color",
                                                section="timeline",
                                                description="What is the color of the parrot's plumage?",
                                                choices=(
                                                    ("Mauve", "Mauve"),
                                                    ("Chartreuse", "Chartreuse"),
                                                    ("Magenta", "Magenta"),
                                                    ("Pink", "Pink"),
                                                    ("Norwegian Blue", "Norwegian Blue"),
                                                    ("Yellow Ochre", "Yellow Ochre")))

        PreferencesDialog.add_toggle_preference('aTogglePreference',
                                                label="Test Toggle",
                                                section="timeline",
                                                description="Test the toggle widget")

        PreferencesDialog.add_font_preference('aFontPreference',
                                              label="Foo Font",
                                              section="timeline",
                                              description="Test the font widget")



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (C) 2020 Andrew Hazel, Thomas Braccia, Troy Ogden, Robert Kirkpatrick
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the pitivi.clip_properties.color module."""
# pylint: disable=protected-access
from unittest import mock

from gi.repository import GES

from tests import common


class ColorPropertiesTest(common.TestCase):
    """Tests for the ColorProperties class."""

    @common.setup_timeline
    @common.setup_clipproperties
    def test_create_hard_coded(self):
        """Exercise creation of a color test clip."""
        self.project.pipeline.get_position = mock.Mock(return_value=0)

        self.clipproperties.create_color_clip_cb(None)
        clips = self.layer.get_clips()
        pattern = clips[0].get_vpattern()
        self.assertEqual(pattern, GES.VideoTestPattern.SOLID_COLOR)

        self.action_log.undo()
        self.assertListEqual(self.layer.get_clips(), [])

        self.action_log.redo()
        self.assertListEqual(self.layer.get_clips(), clips)

    @common.setup_timeline
    @common.setup_clipproperties
    def test_color_change(self):
        """Exercise the changing of colors for color clip."""
        self.project.pipeline.get_position = mock.Mock(return_value=0)

        self.clipproperties.create_color_clip_cb(None)

        mainloop = common.create_main_loop()
        mainloop.run(until_empty=True)

        color_expander = self.clipproperties.color_expander
        color_picker_mock = mock.Mock()
        color_picker_mock.calculate_argb.return_value = 1 << 24 | 2 << 16 | 3 << 8 | 4
        color_expander._color_picker_value_changed_cb(color_picker_mock)
        color = color_expander.source.get_child_property("foreground-color")[1]
        self.assertEqual(color, 0x1020304)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2020, Guy Richard <guy.richard99@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the pitivi.viewer.viewer module."""
# pylint: disable=protected-access,no-self-use
from pitivi.project import ProjectManager
from pitivi.viewer.viewer import ViewerContainer
from tests import common


class TestViewerContainer(common.TestCase):

    def test_toggle_safe_areas_action(self):
        """Checks the effect of the toggle_safe_areas_action."""
        app = common.create_pitivi_mock()
        app.project_manager = ProjectManager(app)
        viewer_container = ViewerContainer(app)

        project = app.project_manager.new_blank_project()
        viewer_container.set_project(project)
        self.assertFalse(viewer_container.overlay_stack.safe_areas_overlay.get_visible())

        viewer_container.toggle_safe_areas_action.activate()
        self.assertTrue(viewer_container.overlay_stack.safe_areas_overlay.get_visible())

        viewer_container.toggle_safe_areas_action.activate()
        self.assertFalse(viewer_container.overlay_stack.safe_areas_overlay.get_visible())



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2016, Alex Băluț <alexandru.balut@gmail.com>
# Copyright (c) 2017, Suhas Nayak <suhas2go@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the effects module."""
# pylint: disable=attribute-defined-outside-init,protected-access
import os

from gi.repository import GES
from gi.repository import Gst
from gi.repository import Gtk

from pitivi.effects import AUDIO_EFFECT
from pitivi.effects import EffectInfo
from pitivi.effects import EffectsPropertiesManager
from pitivi.effects import PROPS_TO_IGNORE
from pitivi.effects import VIDEO_EFFECT
from pitivi.utils.widgets import DynamicWidget
from pitivi.utils.widgets import GstElementSettingsWidget
from pitivi.utils.widgets import NumericWidget
from tests import common


class EffectInfoTest(common.TestCase):
    """Tests for the EffectInfo class."""

    def test_bin_description(self):
        """Checks the bin_description property."""
        effect_info = EffectInfo("name", None, None, None, None)
        self.assertEqual(effect_info.bin_description, "name")

        effect_info = EffectInfo("glname", None, None, None, None)
        self.assertEqual(effect_info.bin_description, "glupload ! glname ! gldownload")

    def test_name_from_bin_description(self):
        """Checks the name_from_bin_description method."""
        self.assertEqual(EffectInfo.name_from_bin_description("name"), "name")
        self.assertEqual(EffectInfo.name_from_bin_description("glupload ! glname ! gldownload"), "glname")

    def test_good_for_track_element(self):
        """Checks the good_for_track_element method."""
        uri = common.get_sample_uri("tears_of_steel.webm")
        asset = GES.UriClipAsset.request_sync(uri)
        ges_clip = asset.extract()

        # Add the clip to a timeline so it gets tracks.
        ges_timeline = common.create_timeline_container().timeline.ges_timeline
        ges_timeline.append_layer()
        ges_layer, = ges_timeline.get_layers()
        ges_layer.add_clip(ges_clip)
        track_elements = {track_element.get_track_type(): track_element
                          for track_element in ges_clip.get_children(recursive=True)}
        audio_track_element = track_elements[GES.TrackType.AUDIO]
        video_track_element = track_elements[GES.TrackType.VIDEO]

        effect_info = EffectInfo(None, AUDIO_EFFECT, None, None, None)
        self.assertTrue(effect_info.good_for_track_element(audio_track_element))
        self.assertFalse(effect_info.good_for_track_element(video_track_element))

        effect_info = EffectInfo(None, VIDEO_EFFECT, None, None, None)
        self.assertFalse(effect_info.good_for_track_element(audio_track_element))
        self.assertTrue(effect_info.good_for_track_element(video_track_element))


class EffectsPropertiesManagerTest(common.TestCase):
    """Tests for the EffectsPropertiesManager class."""

    def create_alpha_widget_cb(self, unused_manager, unused_container, unused_effect, widgets):
        """Handles the request to create an effect widget."""
        self.builder = Gtk.Builder()
        path = os.path.join(os.path.dirname(__file__), "plugins", "test_alpha.ui")
        self.builder.add_objects_from_file(path, widgets)
        self.element_settings_widget.map_builder(self.builder)
        return self.builder.get_object("GstAlpha::black-sensitivity")

    def _register_alpha_widget(self, widgets):
        """Sets up an EffectsPropertiesManager instance to create custom effect UI."""
        self.alpha_effect = GES.Effect.new("alpha")
        self.prop_name = "black-sensitivity"
        _, _, self.prop = self.alpha_effect.lookup_child(self.prop_name)

        self.effects_prop_manager = EffectsPropertiesManager(self.app)
        self.effects_prop_manager.connect("create-widget", self.create_alpha_widget_cb, widgets)
        self.element_settings_widget = GstElementSettingsWidget(self.alpha_effect, PROPS_TO_IGNORE)

        self.effects_prop_manager.emit("create-widget", self.element_settings_widget, self.alpha_effect)
        self.effects_prop_manager._connect_all_widget_callbacks(self.element_settings_widget, self.alpha_effect)
        self.effects_prop_manager._post_configuration(self.alpha_effect, self.element_settings_widget)

    def test_wrapping(self):
        """Checks UI updating results in updating the effect."""
        self.app = common.create_pitivi_mock()
        self._register_alpha_widget(("black_sens_adjustment", "GstAlpha::black-sensitivity"))

        # Check if the widget is wrapped correctly
        wrapped_spin_button = self.element_settings_widget.properties[self.prop]
        self.assertTrue(isinstance(wrapped_spin_button, DynamicWidget))
        self.assertTrue(isinstance(wrapped_spin_button, NumericWidget))

        # Check if the wrapper has the correct default value
        self.assertEqual(self.prop.default_value, wrapped_spin_button.get_widget_default())

        # Check if the callbacks are functioning
        value = (1 + self.prop.default_value) % self.prop.maximum
        wrapped_spin_button.set_widget_value(value)
        self.assertEqual(wrapped_spin_button.get_widget_value(), value)
        _, prop_value = self.alpha_effect.get_child_property(self.prop_name)
        self.assertEqual(prop_value, value)

    def test_prop_keyframe(self):
        """Checks the keyframe button effect."""
        uri = common.get_sample_uri("tears_of_steel.webm")
        asset = GES.UriClipAsset.request_sync(uri)
        ges_clip = asset.extract()

        # Add the clip to a timeline so it gets tracks.
        timeline = common.create_timeline_container()
        self.app = timeline.app
        ges_timeline = timeline.ges_timeline
        ges_timeline.append_layer()
        ges_layer, = ges_timeline.get_layers()
        ges_layer.add_clip(ges_clip)

        self._register_alpha_widget(
            ("black_sens_adjustment", "GstAlpha::black-sensitivity", "GstAlpha::black-sensitivity::keyframe"))
        ges_clip.add(self.alpha_effect)
        track_element = self.element_settings_widget._GstElementSettingsWidget__get_track_element_of_same_type(
            self.alpha_effect)
        prop_keyframe_button = \
            list(self.element_settings_widget._GstElementSettingsWidget__widgets_by_keyframe_button.keys())[0]

        # Control the self.prop property on the timeline
        prop_keyframe_button.set_active(True)
        self.assertEqual(track_element.ui_element._TimelineElement__controlled_property, self.prop)
        # Revert to controlling the default property
        prop_keyframe_button.set_active(False)
        self.assertNotEqual(track_element.ui_element._TimelineElement__controlled_property, self.prop)

    def test_prop_reset(self):
        """Checks the reset button resets the property."""
        self.app = common.create_pitivi_mock()
        self._register_alpha_widget(
            ("black_sens_adjustment", "GstAlpha::black-sensitivity", "GstAlpha::black-sensitivity::reset", "image1"))
        wrapped_spin_button = self.element_settings_widget.properties[self.prop]
        _, prop_value = self.alpha_effect.get_child_property(self.prop_name)
        self.assertEqual(self.prop.default_value, prop_value)
        self.assertEqual(self.prop.default_value, wrapped_spin_button.get_widget_value())

        # Set the property value to a different value than the default
        wrapped_spin_button.set_widget_value((1 + self.prop.default_value) % self.prop.maximum)
        _, prop_value = self.alpha_effect.get_child_property(self.prop_name)
        self.assertEqual(prop_value, (1 + self.prop.default_value) % self.prop.maximum)

        # Reset the value of the property to default
        prop_reset_button = \
            list(self.element_settings_widget._GstElementSettingsWidget__widgets_by_reset_button.keys())[0]
        prop_reset_button.clicked()
        _, prop_value = self.alpha_effect.get_child_property(self.prop_name)
        self.assertEqual(self.prop.default_value, prop_value)
        self.assertEqual(self.prop.default_value, wrapped_spin_button.get_widget_value())

    def test_dependent_properties(self):
        """Checks dependent properties updating is handled correctly."""
        mainloop = common.create_main_loop()
        app = common.create_pitivi()
        app.project_manager.new_blank_project()
        manager = EffectsPropertiesManager(app)

        called = False

        def set_child_property(prop_name, value):
            nonlocal called
            called = True

            self.assertEqual(prop_name, "aspect-ratio")
            GES.Effect.set_child_property(effect, prop_name, value)

            # When setting the aspect-ratio property, and the stars align,
            # the effect also changes the left/right properties.
            # Here we simulate the updating of the dependent properties.
            GES.Effect.set_child_property(effect, "left", 100)
            GES.Effect.set_child_property(effect, "right", 100)

        effect = GES.Effect.new("aspectratiocrop")
        effect.set_child_property = set_child_property

        effect_widget = manager.get_effect_configuration_ui(effect)

        widgets = {prop.name: widget
                   for prop, widget in effect_widget.properties.items()}
        # Simulate the user choosing an aspect-ratio.
        widgets["aspect-ratio"].set_widget_value(Gst.Fraction(4, 3))

        mainloop.run(until_empty=True)

        self.assertTrue(called)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2019, Alex Băluț <alexandru.balut@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the transitions module."""
from pitivi.transitions import TransitionsListWidget
from tests import common


class TransitionsListWidgetTest(common.TestCase):
    """Tests for the TransitionsListWidget class."""

    def test_transition_types_loaded(self):
        """Checks the transition types are properly detected."""
        app = common.create_pitivi_mock()
        widget = TransitionsListWidget(app)
        mainloop = common.create_main_loop()
        mainloop.run(until_empty=True)
        self.assertGreater(len(widget.storemodel), 10)



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2016, Lubosz Sarnecki <lubosz.sarnecki@collabora.co.uk>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
import numpy
from gi.repository import Gdk
from gi.repository import GES
from gi.repository import GLib
from gi.repository import Gtk

from pitivi.utils.loggable import Loggable
from pitivi.viewer.move_scale_overlay import MoveScaleOverlay
from pitivi.viewer.safe_areas_overlay import SafeAreasOverlay
from pitivi.viewer.title_overlay import TitleOverlay


class OverlayStack(Gtk.Overlay, Loggable):
    """Manager for the viewer overlays."""

    def __init__(self, app, sink_widget, guidelines_overlay):
        Gtk.Overlay.__init__(self)
        Loggable.__init__(self)
        self.__overlays = {}
        self.__visible_overlays = []
        self.__hide_all_overlays = False
        self.__last_allocation = None
        self.app = app
        self.window_size = numpy.array([1, 1])
        self.click_position = None
        self.hovered_overlay = None
        self.selected_overlay = None
        self.add_events(Gdk.EventMask.BUTTON_PRESS_MASK |
                        Gdk.EventMask.BUTTON_RELEASE_MASK |
                        Gdk.EventMask.POINTER_MOTION_MASK |
                        Gdk.EventMask.SCROLL_MASK |
                        Gdk.EventMask.ENTER_NOTIFY_MASK |
                        Gdk.EventMask.LEAVE_NOTIFY_MASK |
                        Gdk.EventMask.ALL_EVENTS_MASK)
        self.add(sink_widget)
        self.connect("size-allocate", self.__size_allocate_cb)

        # Whether to show the percent of the size relative to the project size.
        # It is set to false initially because the viewer gets resized
        # while the project is loading and we don't want to show the percent
        # in this case.
        self.__show_resize_status = False
        # ID of resizing timeout callback, so it can be delayed.
        self.__resizing_id = 0
        self.revealer = Gtk.Revealer(transition_type=Gtk.RevealerTransitionType.CROSSFADE)
        self.resize_status = Gtk.Label(name="resize_status")
        self.revealer.add(self.resize_status)
        self.add_overlay(self.revealer)

        self.guidelines_overlay = guidelines_overlay
        self.add_overlay(guidelines_overlay)

        self.safe_areas_overlay = SafeAreasOverlay(self)
        self.add_overlay(self.safe_areas_overlay)

        sink_widget.connect("size-allocate", self.__sink_widget_size_allocate_cb)

    def __size_allocate_cb(self, widget, rectangle):
        self.window_size = numpy.array([rectangle.width,
                                        rectangle.height])
        for overlay in self.__overlays.values():
            overlay.update_from_source()

    def __overlay_for_source(self, source):
        if source in self.__overlays:
            return self.__overlays[source]

        if isinstance(source, GES.TitleSource):
            overlay = TitleOverlay(self, source)
        else:
            overlay = MoveScaleOverlay(self, self.app.action_log, source)
        self.add_overlay(overlay)
        self.__overlays[source] = overlay
        return overlay

    def do_event(self, event):
        if event.type == Gdk.EventType.BUTTON_RELEASE:
            cursor_position = numpy.array([event.x, event.y])
            self.click_position = None
            if self.selected_overlay:
                self.selected_overlay.on_button_release(cursor_position)
            # reset the cursor if we are outside of the viewer
            if (cursor_position < numpy.zeros(2)).any() or (cursor_position > self.window_size).any():
                self.reset_cursor()
        elif event.type == Gdk.EventType.LEAVE_NOTIFY and event.mode == Gdk.CrossingMode.NORMAL:
            # If we have a click position, the user is dragging, so we don't want to lose focus and return
            if isinstance(self.click_position, numpy.ndarray):
                return False
            for overlay in self.__overlays.values():
                overlay.unhover()
            self.reset_cursor()
        elif event.type == Gdk.EventType.BUTTON_PRESS:
            self.click_position = numpy.array([event.x, event.y])
            if self.hovered_overlay:
                self.hovered_overlay.on_button_press()
            elif self.selected_overlay:
                self.selected_overlay.on_button_press()
        elif event.type == Gdk.EventType.MOTION_NOTIFY:
            cursor_position = numpy.array([event.x, event.y])

            if isinstance(self.click_position, numpy.ndarray):
                if self.selected_overlay:
                    self.selected_overlay.on_motion_notify(cursor_position)
            else:

                # Prioritize Handles
                if isinstance(self.selected_overlay, MoveScaleOverlay):
                    if self.selected_overlay.on_hover(cursor_position):
                        if self.selected_overlay.hovered_handle:
                            self.hovered_overlay = self.selected_overlay
                            return False

                for overlay in self.__visible_overlays:
                    if overlay.on_hover(cursor_position):
                        self.hovered_overlay = overlay
                        break
                if not self.hovered_overlay:
                    self.reset_cursor()
        elif event.type == Gdk.EventType.SCROLL:
            # TODO: Viewer zoom
            pass
        return True

    def set_current_sources(self, sources):
        """Sets the sources at the playhead."""
        self.__visible_overlays = []
        # check if source has instanced viewer
        for source in sources:
            overlay = self.__overlay_for_source(source)
            self.__visible_overlays.append(overlay)
        # check if viewer should be visible
        if not self.__hide_all_overlays:
            for source, overlay in self.__overlays.items():
                if source in sources:
                    overlay.show()
                else:
                    overlay.hide()

    def update(self, source):
        self.__overlays[source].update_from_source()

    def select(self, source):
        """Specifies the selected source between the sources at the playhead."""
        if not source:
            self.selected_overlay = None
            return

        self.selected_overlay = self.__overlay_for_source(source)
        self.selected_overlay.queue_draw()

    def hide_overlays(self):
        if self.__hide_all_overlays:
            # The overlays are already hidden.
            return

        self.guidelines_overlay.hide()
        for overlay in self.__visible_overlays:
            overlay.hide()
        self.__hide_all_overlays = True

    def show_overlays(self):
        if not self.__hide_all_overlays:
            # The overlays are already visible.
            return

        self.guidelines_overlay.show()
        for overlay in self.__visible_overlays:
            overlay.show()
        self.__hide_all_overlays = False

    def set_cursor(self, name):
        cursor = None
        display = Gdk.Display.get_default()
        try:
            cursor = Gdk.Cursor.new_from_name(display, name)
        except TypeError:
            self.warning("Cursor '%s' not found.", name)
        self.app.gui.get_window().set_cursor(cursor)

    def reset_cursor(self):
        self.app.gui.get_window().set_cursor(None)

    def get_drag_distance(self, cursor_position):
        return cursor_position - self.click_position

    def get_normalized_drag_distance(self, cursor_position):
        return self.get_drag_distance(cursor_position) / self.window_size

    def get_normalized_cursor_position(self, cursor_position):
        return cursor_position / self.window_size

    def enable_resize_status(self, enabled):
        self.__show_resize_status = enabled

    def __sink_widget_size_allocate_cb(self, unused_widget, allocation):
        previous_allocation = self.__last_allocation
        self.__last_allocation = (allocation.width, allocation.height)
        if previous_allocation == self.__last_allocation:
            # The allocation did not actually change. Ignore the event.
            return

        if not self.__show_resize_status:
            return

        if not self.revealer.get_reveal_child():
            self.revealer.set_transition_duration(10)
            self.revealer.set_reveal_child(True)
            self.hide_overlays()

        video_width = self.app.project_manager.current_project.videowidth
        percent = int(allocation.width / video_width * 100)
        self.resize_status.set_text("{}%".format(percent))

        # Add timeout function to hide the resize percent.
        if self.__resizing_id:
            GLib.source_remove(self.__resizing_id)
        self.__resizing_id = GLib.timeout_add(1000, self.__resizing_timeout_cb, None)

    def __resizing_timeout_cb(self, unused_data):
        self.__resizing_id = 0
        self.revealer.set_transition_duration(500)
        self.revealer.set_reveal_child(False)
        self.show_overlays()
        return False



# -*- coding: utf-8 -*-
# Pitivi video editor
# Copyright (c) 2020, Guy Richard <guy.richard99@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, see <http://www.gnu.org/licenses/>.
"""Tests for the pitivi.viewer.safe_areas_overlay module."""
# pylint: disable=protected-access,no-self-use
from pitivi.viewer.safe_areas_overlay import SafeAreasOverlay
from tests import common


class TestSafeAreasOverlay(common.TestCase):
    """Tests for the SafeAreasOverlay class."""

    def test_compute_rect(self):
        """Checks the safe areas Cairo rect calculation."""
        self.assertEqual(SafeAreasOverlay._compute_rect(200, 100, 1, 0.5), (0.5, 25.5, 200, 50))
        self.assertEqual(SafeAreasOverlay._compute_rect(200, 100, 0.5, 1), (50.5, 0.5, 100, 100))
